@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * The ARAnyM BetaDOS driver.
  *
  * 2001/2002 STan
@@ -13,22 +11,12 @@
  *
  **/
 
-#include <mintbind.h>
-#include <mint/basepage.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "mint/mint.h"
-#include "mint/filedesc.h"
-#include "mint/kerinfo.h"
-
-#include "../hostfs_xfs.h"
-#include "../hostfs_dev.h"
-
-#include "mintproc.h"
-#include "mintfake.h"
-
+#include "hostfs.h"
 #include "nf_ops.h"
+#include <osbind.h>
+#include "hostfs/hostfs_dev.h"
+#include "hostfs/hostfs_xfs.h"
+#include "global.h"
 
 
 #define DEVNAME "ARAnyM Host Filesystem"
@@ -37,17 +25,16 @@
 char DriverName[] = DEVNAME" "VERSION;
 long ldp;
 
+long _cdecl (*nf_call)(long id, ...) = 0L;
 
 void _cdecl ShowBanner( void );
 void* _cdecl InitDevice( long bosDevID, long dosDevID );
-long set_cookie (ulong tag, ulong val);
-ulong get_cookie (ulong tag);
 
 /* Diverse Utility-Funktionen */
 
 void _cdecl ShowBanner( void )
 {
-    Cconws (
+	(void) Cconws (
             "\r\n\033p "DEVNAME" "VERSION" \033q "
             "\r\nCopyright (c) ARAnyM Development Team, "__DATE__"\r\n"
             );
@@ -59,7 +46,7 @@ struct cookie
 	long value;
 };
 
-ulong get_cookie (ulong tag)
+static ulong get_cookie (ulong tag)
 {
 	struct cookie *cookie = *(struct cookie **)0x5a0;
 	if (!cookie) return 0;
@@ -72,33 +59,46 @@ ulong get_cookie (ulong tag)
 	return 0;
 }
 
-long set_cookie (ulong tag, ulong val)
+static long set_cookie (ulong tag, ulong val)
 {
 	struct cookie *cookie = *(struct cookie **)0x5a0;
-	if (!cookie) return 0;
+	long count, size;
 
-	while (cookie->tag) cookie++;
+	if (!cookie) return 0;
+	
+	count = 0;
+	while (cookie->tag)
+	{
+		cookie++;
+		count++;
+	}
+	count += 2;
+	size = cookie->value;
+	if (count >= size)
+		return 0;
 	cookie->tag = tag;
 	cookie->value = val;
+	cookie++;
+	cookie->tag = 0;
+	cookie->value = size;
 	return 1;
 }
 
 
-/*
- * global kerinfo structure
- */
-struct kerinfo *KERNEL;
-
-
 void* _cdecl InitDevice( long bosDevID, long dosDevID )
 {
 	char mountPoint[] = "A:";
-	mountPoint[0] += (dosDevID = (dosDevID&0x1f)); // mask out bad values of the dosDevID
+	mountPoint[0] += (dosDevID = (dosDevID&0x1f)); /* mask out bad values of the dosDevID */
+	
+	/* check the NF HOSTFS avialability */
+	if ( ! hostfs_init() ) {
+		return (void*)-1;
+	}
 
 	/*
 	 * Hack to get the drive table the same for all hostfs.dos
 	 * instances loaded by BetaDOS into memory.
-         *
+     *
 	 * Note: This is definitely not MP friendly, but FreeMiNT
 	 *       doesn't support B(M)etaDOS anyway, so: Do not do
 	 *       this when using 'MiNT' or 'MagX' it crashes then.
@@ -119,20 +119,12 @@ void* _cdecl InitDevice( long bosDevID, long dosDevID )
 	 */
 	DEBUG(("InitDevice: %s [dosDev=%ld, bosDev=%ld] addr: %lx", mountPoint, dosDevID, bosDevID, &hostfs_filesys ));
 
-	/* initialize Native Features */
-	kernelinfo.nf_ops = nf_init();
-	KERNEL = &kernelinfo;
-
-	/* check the NF HOSTFS avialability */
-	if ( ! hostfs_init() ) {
-		return (void*)-1;
-	}
-
 	/* map the BetaDOS drive to some bosDrive | 0x6000 so that the mapping would
 	   not colide with the MiNT one */
 	fs_native_init( dosDevID | 0x6000, mountPoint, "/tmp", 1,
 					&hostfs_filesys, &hostfs_fs_devdrv );
-
+	timezone = hostfs_filesys.res1;
+	
 	hostfs_filesys.root( dosDevID | 0x6000, &curproc->p_cwd->root[dosDevID] );
 
 #ifdef DEBUG_INFO
@@ -148,7 +140,6 @@ void* _cdecl InitDevice( long bosDevID, long dosDevID )
 
 
 /**
- * $Log$
  * Revision 1.12  2006/02/06 20:58:17  standa
  * Sync with the FreeMiNT CVS. The make.sh now only builds the BetaDOS
  * hostfs.dos.
