@@ -9,44 +9,48 @@
 #ifndef JPGDH_H
 #define JPGDH_H
 
-#ifdef ARANYM_NFJPEG
-#define	VOID_PTR	uint32
-#define	UCHAR_PTR	uint32
-#define FUNC_PTR(x,y)	uint32 x
-#define LONG		int32
-#define ULONG		uint32
-#else
+#if defined(__atarist__) || defined(__TOS__)
 #define VOID_PTR	void *
 #define UCHAR_PTR	unsigned char *
 #define FUNC_PTR(x,y)	short (*x)(y)
-#define LONG		long
-#define ULONG		unsigned long
+#define _LONG		long
+#define _ULONG		unsigned long
+#else
+/* otherwise in emulator */
+#define	VOID_PTR	uint32
+#define	UCHAR_PTR	uint32
+#define FUNC_PTR(x,y)	uint32 x
+#define _LONG		int32
+#define _ULONG		uint32
 #endif
 
-typedef struct _JPGD_STRUCT {
+typedef struct _JPGD_STRUCT JPGD_STRUCT;
+typedef JPGD_STRUCT	*JPGD_PTR;
+
+struct _JPGD_STRUCT {
 	VOID_PTR	InPointer;							/* JPEG Image Pointer */
 	VOID_PTR	OutPointer;							/* Output Buffer/Filename Pointer (see OutFlag) */
-	LONG	InSize;									/* JPEG Image Size (Bytes) */
-	LONG	OutSize;								/* Output Image Size (Bytes) */
+	_LONG	InSize;									/* JPEG Image Size (Bytes) */
+	_LONG	OutSize;								/* Output Image Size (Bytes) */
 	short	InComponents;							/* JPEG Image Components Number (1->4) */
 	short	OutComponents;							/* Output Components Number (1->4) */
 	short	OutPixelSize;							/* Output Pixel Size (1->4) */
 	short	OutFlag;								/* 0 (RAM Output) / -1 (Disk Output) */
 	short	XLoopCounter;							/* Number of MCUs per Row */
 	short	YLoopCounter;							/* Number of MCUs per Column */
-	FUNC_PTR(Create,struct _JPGD_STRUCT *);			/* Pointer to User Routine / NULL */
-	FUNC_PTR(Write,struct _JPGD_STRUCT *);			/* Pointer to User Routine / NULL */
-	FUNC_PTR(Close,struct _JPGD_STRUCT *);			/* Pointer to User Routine / NULL */
-	FUNC_PTR(SigTerm,struct _JPGD_STRUCT *);		/* Pointer to User Routine / NULL */
+	FUNC_PTR(Create, JPGD_STRUCT *);				/* Pointer to User Routine / NULL */
+	FUNC_PTR(Write, JPGD_STRUCT *);					/* Pointer to User Routine / NULL */
+	FUNC_PTR(Close, JPGD_STRUCT *);					/* Pointer to User Routine / NULL */
+	FUNC_PTR(SigTerm, JPGD_STRUCT *);				/* Pointer to User Routine / NULL */
 	UCHAR_PTR	Comp1GammaPtr;						/* Component 1 Gamma Table / NULL */
 	UCHAR_PTR	Comp2GammaPtr;						/* Component 2 Gamma Table / NULL */
 	UCHAR_PTR	Comp3GammaPtr;						/* Component 3 Gamma Table / NULL */
 	UCHAR_PTR	Comp4GammaPtr;						/* Component 4 Gamma Table / NULL */
-	FUNC_PTR(UserRoutine,struct _JPGD_STRUCT *);	/* Pointer to User Routine (Called during Decompression) / NULL */
+	FUNC_PTR(UserRoutine, JPGD_STRUCT *);			/* Pointer to User Routine (Called during Decompression) / NULL */
 	VOID_PTR	OutTmpPointer;						/* Current OutPointer / Temporary Disk Buffer Pointer (see OutFlag) */
 	short	MCUsCounter;							/* Number of MCUs not Decoded */
 	short	OutTmpHeight;							/* Number of Lines in OutTmpPointer */
-	LONG	User[2];								/* Free, Available for User */
+	_LONG	User[2];								/* Free, Available for User */
 	short	OutHandle;								/* 0 / Output File Handle (see OutFlag) */
 
 	/* Output image MFDB */
@@ -62,15 +66,14 @@ typedef struct _JPGD_STRUCT {
 
 	/* Official structure stop here, what follows is decoder-dependant */
 
-	ULONG handle;	/* ARAnyM image handle */
-} __attribute__((packed)) JPGD_STRUCT;
-typedef JPGD_STRUCT	*JPGD_PTR;
+	_ULONG handle;	/* ARAnyM image handle */
+} __attribute__((packed));
 
-#define	JPGD_MAGIC	'_JPD'
+#define	JPGD_MAGIC	0x5F4A5044L /* '_JPD' */
 #define	JPGD_VERSION	1
 
 #undef NOERROR
-enum JPGD_ENUM {
+enum _JPGD_ENUM {
 	NOERROR=0,			/* File correctly uncompressed */
 	UNKNOWNFORMAT,		/* File is not JFIF (Error) */
 	INVALIDMARKER,		/* Reserved CCITT Marker Found (Error) */
@@ -113,9 +116,7 @@ enum JPGD_ENUM {
 	DRIVERCLOSED,		/* Driver is Already Closed. */
 	ENDOFIMAGE			/* Stop Decoding (Internal Message, Should Never Appear) */
 };
-#ifndef ARANYM_NFJPEG
 typedef	long	JPGD_ENUM;
-#endif
 
 typedef struct {
 	long		JPGDVersion;
