@@ -23,10 +23,14 @@
 # endif
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
  * load & initialize TinyGL
  */
-LDG *ldg_load_tiny_gl(const char *libname, _WORD *gl);
+struct gl_public *ldg_load_tiny_gl(const char *libname, _WORD *gl);
 
 /*
  * init TinyGL from already loaded lib
@@ -36,7 +40,11 @@ int ldg_init_tiny_gl(LDG *lib);
 /*
  * unload TinyGL
  */
-void ldg_unload_tiny_gl(LDG *lib, _WORD *gl);
+void ldg_unload_tiny_gl(struct gl_public *pub, _WORD *gl);
+
+#ifdef __cplusplus
+}
+#endif
 
 
 
@@ -49,7 +57,7 @@ extern "C" {
 #endif
 
 #ifndef APIENTRY
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(_WIN32) || defined(__WIN32__) || defined(__CYGWIN__)
 #define APIENTRY __stdcall
 #elif defined(__PUREC__)
 #define APIENTRY __CDECL
@@ -63,6 +71,9 @@ extern "C" {
 #ifndef GLAPIENTRY
 #define GLAPIENTRY APIENTRY
 #endif
+#ifndef APIENTRYP
+#define APIENTRYP APIENTRY *
+#endif
 #ifndef GLAPI
 #define GLAPI extern
 #endif
@@ -672,6 +683,7 @@ typedef void GLvoid;
 #define GL_EXTENSIONS						0x1F03
 
 /* Errors */
+#define GL_NO_ERROR                         0
 #define GL_INVALID_VALUE					0x0501
 #define GL_INVALID_ENUM						0x0500
 #define GL_INVALID_OPERATION				0x0502
@@ -958,9 +970,21 @@ GLAPI void GLAPIENTRY glPolygonOffset(GLfloat factor, GLfloat units);
 #endif /* GL_VERSION_1_1 */
 
 #ifndef GL_VERSION_1_5
-#define GL_VERSION_1_5 1
-typedef ptrdiff_t GLintptr;
+#ifndef __GLintptr_defined
+#include <stddef.h>
+#ifdef __APPLE__
+typedef intptr_t GLsizeiptr;
+typedef intptr_t GLintptr;
+#else
 typedef ptrdiff_t GLsizeiptr;
+typedef ptrdiff_t GLintptr;
+#endif
+#define __GLintptr_defined
+#endif
+#endif
+
+#ifdef __NFOSMESA_H__
+#include <GL/glext.h>
 #endif
 
 #ifndef GL_VERSION_2_0
@@ -970,9 +994,14 @@ typedef char GLchar;
 
 #ifndef GL_ARB_vertex_buffer_object
 #define GL_ARB_vertex_buffer_object 1
+#ifdef __APPLE__
+typedef intptr_t GLsizeiptrARB;
+typedef intptr_t GLintptrARB;
+#else
 typedef ptrdiff_t GLsizeiptrARB;
 typedef ptrdiff_t GLintptrARB;
 #endif
+#endif
 
 #ifndef GL_NV_half_float
 #define GL_NV_half_float 1
@@ -1024,8 +1053,12 @@ typedef GLint GLfixed;
 #ifndef GL_ARB_shader_objects
 #define GL_ARB_shader_objects 1
 typedef char GLcharARB;
+#ifdef __APPLE__
+typedef void *GLhandleARB;
+#else
 typedef GLuint GLhandleARB;
 #endif
+#endif
 
 #ifndef GL_KHR_debug
 #define GL_KHR_debug 1
@@ -1043,6 +1076,14 @@ typedef void CALLBACK (*GLDEBUGPROCAMD)(GLuint id,GLenum category, GLenum severi
 #define GL_MESA_program_debug 1
 typedef void CALLBACK (*GLprogramcallbackMESA)(GLenum target, GLvoid *data);
 #endif
+#ifndef GL_EXT_external_buffer
+#define GL_EXT_external_buffer 1
+typedef void *GLeglClientBufferEXT;
+#endif
+#ifndef GL_NV_draw_vulkan_image
+#define GL_NV_draw_vulkan_image 1
+typedef void (APIENTRY *GLVULKANPROCNV)(void);
+#endif
 
 
 /*
@@ -1096,7 +1137,7 @@ struct osmesa_context { int dummy; };
 #endif
 typedef struct osmesa_context *OSMesaContext;
 
-typedef void (*OSMESAproc)(void);
+typedef void (APIENTRY *OSMESAproc)(void);
 
 GLAPI OSMesaContext GLAPIENTRY OSMesaCreateContext( GLenum format, OSMesaContext sharelist );
 GLAPI OSMesaContext GLAPIENTRY OSMesaCreateContextExt( GLenum format, GLint depthBits, GLint stencilBits, GLint accumBits, OSMesaContext sharelist);
@@ -1113,7 +1154,7 @@ GLAPI void GLAPIENTRY OSMesaPostprocess(OSMesaContext osmesa, const char *filter
 
 
 #elif OSMESA_VERSION < OSMESA_VERSION_NUMBER(6, 3, 0)
-typedef void (*OSMESAproc)(void);
+typedef void (APIENTRY *OSMESAproc)(void);
 #endif
 
 /*
@@ -1141,10 +1182,24 @@ typedef GLhalfNV GLhalfNV32;
 #endif
 #endif
 
+/*
+ * Atari-specific structure for the shared libraries
+ */
+struct gl_public {
+	void *libhandle;
+	void *libexec;
+	void *(*m_alloc)(size_t);
+	void (*m_free)(void *);
+};
+
 #ifdef __cplusplus
 }
 #endif
 
+#undef glClearDepth
+#undef glFrustum
+#undef glOrtho
+#undef gluLookAt
 
 /*
  * old LDG functions
@@ -1159,7 +1214,7 @@ GLAPI GLsizei GLAPIENTRY max_width(void);
 GLAPI GLsizei GLAPIENTRY max_height(void);
 GLAPI void GLAPIENTRY swapbuffer(void *buf);
 GLAPI void GLAPIENTRY exception_error(void CALLBACK (*exception)(GLenum param));
-GLAPI void GLAPIENTRY information(void);
+GLAPI void GLAPIENTRY tinyglinformation(void);
 
 GLAPI void GLAPIENTRY glOrthof(GLfloat left, GLfloat right, GLfloat bottom, GLfloat top, GLfloat near_val, GLfloat far_val);
 GLAPI void GLAPIENTRY glFrustumf(GLfloat left, GLfloat right, GLfloat bottom, GLfloat top, GLfloat near_val, GLfloat far_val);
@@ -1444,6 +1499,10 @@ extern struct _gl_tiny gl;
 #undef gluLookAt
 #define gluLookAt gluLookAtf
 
+/* fix bad name */
+#undef information
+#define tinyglinformation gl.information
+
 
 
 /*
@@ -1452,6 +1511,10 @@ extern struct _gl_tiny gl;
 #undef glFinish
 #define glFinish()
 
+/* fix bad name */
+#undef information
+#define tinyglinformation gl.information
+
 /*
  * Functions from OpenGL that are not implemented in TinyGL
  */
