@@ -3,8 +3,8 @@
 ########################## BUILD CPU EMULATION CORE ##########################
 
 mkdir -p "$DERIVED_FILES_DIR" && cd "$DERIVED_FILES_DIR" || exit 1
-MISSING_FILE_CNT=`ls cpuemu.cpp cpudefs.cpp cputbl.h cpustbl.cpp 2>&1 1>/dev/null | wc -l`
-rsync -pogt --stats cpuemu*.cpp cpudefs.cpp cputbl.h cpustbl*.cpp config*.h "$BUILD_DIR" > rsync_output.txt 2>/dev/null
+MISSING_FILE_CNT=`ls cpuemu.cpp cpudefs.cpp cputbl.h cpustbl.cpp cpufunctbl.cpp 2>&1 1>/dev/null | wc -l`
+rsync -pogt --stats cpuemu*.cpp cpudefs.cpp cputbl.h cpustbl*.cpp cpufunctbl*.cpp config*.h "$BUILD_DIR" > rsync_output.txt 2>/dev/null
 RET=$?
 #cat rsync_output.txt
 TRANSFERRED=`cat rsync_output.txt | grep "transferred:" | awk '{print $5}'`
@@ -18,30 +18,43 @@ fi
 # remove old files
 rm -f "$DERIVED_FILES_DIR"/cpu*.*
 
-if [ "x$TARGET_NAME" = "xMacAranym JIT" ]; then
+
+case $TARGET_NAME in
+*JIT*)
   IS_JIT_COMPILE="+"
   JIT_ADDITIONAL_FILES="compemu.cpp compstbl.cpp comptbl.h"
   JIT_TARGET=./compemu.cpp
   echo "JIT CPU generation: $JIT_ADDITIONAL_FILES"
-else
+  ;;
+*)
   echo "no JIT CPU generation"
   IS_JIT_COMPILE="-"
   JIT_ADDITIONAL_FILES=
-fi
+  ;;
+esac
 
 for ARCH in $ARCHS ; do
   CPU_TYPE=$(eval echo $(echo \$CPU_TYPE_$ARCH))
   echo ; echo "Building CPU emulation core for $ARCH"
-  mkdir -p "$TEMP_FILES_DIR/$ARCH"
-  cd "$TEMP_FILES_DIR/$ARCH"
-  cp "$DERIVED_FILES_DIR"/config*.h .
-  make -f "$DERIVED_FILES_DIR/Makefile_$ARCH" top_srcdir="$SOURCE_DIR/.." GEN_DIR=. version_date.h ./cpuemu.cpp $JIT_TARGET || exit 1
+  cd "$DERIVED_FILES_DIR"
+  make -f "Makefile_$ARCH" version_date.h || exit 1
+  cd src/uae_cpu
+  test -d compiler || mkdir compiler
+  make cpuemu.cpp $JIT_TARGET || exit 1
   mv cpuemu.cpp "$DERIVED_FILES_DIR/cpuemu_$ARCH.cpp"
+  mv cpudefs.cpp cputbl.h cpustbl*.cpp $JIT_ADDITIONAL_FILES "$DERIVED_FILES_DIR"
   cat  >> "$DERIVED_FILES_DIR/cpuemu.cpp" << EOF
 #ifdef $CPU_TYPE
 #include "cpuemu_$ARCH.cpp"
 #endif
 
+EOF
+  mv cpufunctbl.cpp "$DERIVED_FILES_DIR/cpufunctbl_$ARCH.cpp"
+  cat  >> "$DERIVED_FILES_DIR/cpufunctbl.cpp" << EOF
+#ifdef $CPU_TYPE
+#include "cpufunctbl_$ARCH.cpp"
+#endif
+
 EOF
 done
 
@@ -60,8 +73,7 @@ EOF
 
 fi
 
-rsync -pogt version_date.h cpudefs.cpp cputbl.h cpustbl*.cpp $JIT_ADDITIONAL_FILES "$DERIVED_FILES_DIR"
 cd "$DERIVED_FILES_DIR"
-rsync -pogt cpuemu*.cpp cpudefs.cpp cputbl.h cpustbl*.cpp $JIT_ADDITIONAL_FILES version_date.h "$BUILD_DIR" || exit 1
+rsync -pogt cpuemu*.cpp cpudefs.cpp cputbl.h cpustbl*.cpp cpufunctbl*.cpp $JIT_ADDITIONAL_FILES version_date.h "$BUILD_DIR" || exit 1
 
-exit 0
\ No newline at end of file
+exit 0
