@@ -32,7 +32,9 @@
  *          20010404 - Arnaldo Carvalho de Melo, use setlocale
  */
 
-//#include <features.h>
+#ifdef __linux__
+#include "linux/libcwrap.h"
+#endif
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/ioctl.h>
@@ -64,8 +66,9 @@ static int set_ip_using(const char *name, unsigned long c, const char *ip, const
     struct sockaddr_in sin;
     char host[128];
 
+    memset(&ifr, 0, sizeof(ifr));
     safe_strncpy(ifr.ifr_name, name, IFNAMSIZ);
-    memset(&sin, 0, sizeof(struct sockaddr));
+    memset(&sin, 0, sizeof(sin));
     sin.sin_family = AF_INET;
 	safe_strncpy(host, ip, (sizeof host));
     if (! inet_aton(host, &sin.sin_addr)) {
@@ -97,6 +100,7 @@ int set_mtu(const char *name, int mtu_size)
 {
     struct ifreq ifr;
 
+    memset(&ifr, 0, sizeof(ifr));
     safe_strncpy(ifr.ifr_name, name, IFNAMSIZ);
 	ifr.ifr_mtu = mtu_size;
 	if (ioctl(skfd, SIOCSIFMTU, &ifr) < 0) {
@@ -111,6 +115,7 @@ static int set_flag(const char *name, short flag)
 {
     struct ifreq ifr;
 
+    memset(&ifr, 0, sizeof(ifr));
     safe_strncpy(ifr.ifr_name, name, IFNAMSIZ);
     if (ioctl (skfd, SIOCGIFFLAGS, &ifr) < 0) {
 		perror("SIOCGIFFLAGS");
