@@ -24,10 +24,14 @@
 #include "sysdeps.h"
 #include "hardware.h"
 #include "cpu_emulation.h"
-#include "memory.h"
+#include "newcpu.h"
+#include "memory-uae.h"
 #include "aradata.h"
 #include "parameters.h"
 
+#define DEBUG 0
+#include "debug.h"
+
 ARADATA::ARADATA(memptr addr, uint32 size) : BASE_IO(addr, size)
 {
 	reset();
@@ -55,6 +59,12 @@ void ARADATA::reset()
 	abase = 0;
 }
 
+void ARADATA::setAbase(memptr new_abase)
+{
+	abase = new_abase;
+	D(bug("ARADATA:: abase=%08x", new_abase));
+}
+
 uint8 ARADATA::handleRead(memptr addr) {
 	addr -= getHWoffset();
 	switch(addr) {
