@@ -24,10 +24,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
+#include "sysdeps.h"
 #include "sdlgui.h"
 #include "file.h"
 #include "ata.h"
@@ -48,144 +45,14 @@ static char part_size[PARTS][7];
 static char part_path[PARTS][80];
 
 /* The disks dialog: */
-enum PARTITIONDLG {
-	box_main,
-	
-	header,
-	
-	box_part0,
-	PART0_BROWSE,
-	PART0_PATH,
-	text_id0,
-	PART0_ID,
-	PART0_PRESENT,
-	PART0_READONLY,
-	PART0_BYTESWAP,
-	text_size0,
-	PART0_SIZE,
-	text_size0mb,
-	PART0_GENERATE,
-
-	box_part1,
-	PART1_BROWSE,
-	PART1_PATH,
-	text_id1,
-	PART1_ID,
-	PART1_PRESENT,
-	PART1_READONLY,
-	PART1_BYTESWAP,
-	text_size1,
-	PART1_SIZE,
-	text_size1mb,
-	PART1_GENERATE,
-
-	box_part2,
-	PART2_BROWSE,
-	PART2_PATH,
-	text_id2,
-	PART2_ID,
-	PART2_PRESENT,
-	PART2_READONLY,
-	PART2_BYTESWAP,
-	text_size2,
-	PART2_SIZE,
-	text_size2mb,
-	PART2_GENERATE,
-
-	box_part3,
-	PART3_BROWSE,
-	PART3_PATH,
-	text_id3,
-	PART3_ID,
-	PART3_PRESENT,
-	PART3_READONLY,
-	PART3_BYTESWAP,
-	text_size3,
-	PART3_SIZE,
-	text_size3mb,
-	PART3_GENERATE,
-
-	text_ide,
-	MAP_IDE0,
-	MAP_IDE1,
-	
-	HELP,
-	APPLY,
-	CANCEL
-};
+#define SDLGUI_INCLUDE_PARTITIONDLG
+#include "sdlgui.sdl"
 
 static int PARTS_PATH[] = { PART0_PATH, PART1_PATH, PART2_PATH, PART3_PATH };
 static int PARTS_PRESENT[] = { PART0_PRESENT, PART1_PRESENT, PART2_PRESENT, PART3_PRESENT };
 static int PARTS_READONLY[] = { PART0_READONLY, PART1_READONLY, PART2_READONLY, PART3_READONLY };
 static int PARTS_BYTESWAP[] = { PART0_BYTESWAP, PART1_BYTESWAP, PART2_BYTESWAP, PART3_BYTESWAP };
 
-static SGOBJ partitiondlg[] = {
-	{SGBOX, SG_BACKGROUND, 0, 0, 0, 76, 25, NULL},
-
-	{SGTEXT, 0, 0, 16, 1, 42, 1, "XHDI Direct Access to Host Disk Partitions"},
-
-	{SGBOX, 0, 0, 1, 3, 74, 3, NULL},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 2, 3, 6, 1, "SCSI0:"},
-	{SGTEXT, 0, 0, 9, 3, 65, 1, NULL},
-	{SGTEXT, 0, 0, 2, 5, 7, 1, "PartID:"},
-	{SGEDITFIELD, 0, 0, 9, 5, 3, 1, part_id[0]},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 14, 5, 8, 1, "Present"},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 26, 5, 8, 1, "ReadOnly"},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 39, 5, 8, 1, "ByteSwap"},
-	{SGTEXT, 0, 0, 52, 5, 5, 1, "Size:"},
-	{SGEDITFIELD, 0, 0, 57, 5, 6, 1, part_size[0]},
-	{SGTEXT, 0, 0, 64, 5, 2, 1, "MB"},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 67, 5, 7, 1, "Create"},
-
-	{SGBOX, 0, 0, 1, 7, 74, 3, NULL},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 2, 7, 6, 1, "SCSI1:"},
-	{SGTEXT, 0, 0, 9, 7, 65, 1, NULL},
-	{SGTEXT, 0, 0, 2, 9, 7, 1, "PartID:"},
-	{SGEDITFIELD, 0, 0, 9, 9, 3, 1, part_id[1]},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 14, 9, 8, 1, "Present"},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 26, 9, 8, 1, "ReadOnly"},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 39, 9, 8, 1, "ByteSwap"},
-	{SGTEXT, 0, 0, 52, 9, 5, 1, "Size:"},
-	{SGEDITFIELD, 0, 0, 57, 9, 6, 1, part_size[1]},
-	{SGTEXT, 0, 0, 64, 9, 2, 1, "MB"},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 67, 9, 7, 1, "Create"},
-
-	{SGBOX, 0, 0, 1, 11, 74, 3, NULL},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 2, 11, 6, 1, "SCSI2:"},
-	{SGTEXT, 0, 0, 9, 11, 65, 1, NULL},
-	{SGTEXT, 0, 0, 2, 13, 7, 1, "PartID:"},
-	{SGEDITFIELD, 0, 0, 9, 13, 3, 1, part_id[2]},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 14, 13, 8, 1, "Present"},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 26, 13, 8, 1, "ReadOnly"},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 39, 13, 8, 1, "ByteSwap"},
-	{SGTEXT, 0, 0, 52, 13, 5, 1, "Size:"},
-	{SGEDITFIELD, 0, 0, 57, 13, 6, 1, part_size[2]},
-	{SGTEXT, 0, 0, 64, 13, 2, 1, "MB"},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 67, 13, 7, 1, "Create"},
-
-	{SGBOX, 0, 0, 1, 15, 74, 3, NULL},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 2, 15, 6, 1, "SCSI3:"},
-	{SGTEXT, 0, 0, 9, 15, 65, 1, NULL},
-	{SGTEXT, 0, 0, 2, 17, 7, 1, "PartID:"},
-	{SGEDITFIELD, 0, 0, 9, 17, 3, 1, part_id[3]},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 14, 17, 8, 1, "Present"},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 26, 17, 8, 1, "ReadOnly"},
-	{SGCHECKBOX, SG_SELECTABLE, 0, 39, 17, 8, 1, "ByteSwap"},
-	{SGTEXT, 0, 0, 52, 17, 5, 1, "Size:"},
-	{SGEDITFIELD, 0, 0, 57, 17, 6, 1, part_size[3]},
-	{SGTEXT, 0, 0, 64, 17, 2, 1, "MB"},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 67, 17, 7, 1, "Create"},
-
-	{SGTEXT, 0, SG_DISABLED, 2, 20, 32, 1, "Map IDE disk drives as XHDI IDE:"},
-	{SGCHECKBOX, SG_SELECTABLE, SG_SELECTED | SG_DISABLED, 36, 20, 16, 1, "IDE0 (Master)"},
-	{SGCHECKBOX, SG_SELECTABLE, SG_SELECTED | SG_DISABLED, 54, 20, 15, 1, "IDE1 (Slave)"},
-
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 2, 23, 6, 1, "Help"},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT | SG_DEFAULT, 0, 54, 23, 8, 1, "Apply"},
-	{SGBUTTON, SG_SELECTABLE | SG_EXIT, 0, 66, 23, 8, 1, "Cancel"},
-	{-1, 0, 0, 0, 0, 0, 0, NULL}
-};
-
 static const char *HELP_TEXT =
 	"For creating new partition image click on the [SCSIx] and select a file (or type in a new filename).\n"
 	"Then set the desired partition size in MegaBytes.\n"
@@ -339,6 +206,35 @@ int DlgPartition::processDialogMain(void)
 			selectPartPath(3);
 			break;
 
+		case PART0_CLEAR:
+			part_path[0][0] = '\0';
+			part_size[0][0] = '\0';
+			setSelected(PARTS_PRESENT[0], false);
+			setSelected(PARTS_READONLY[0], false);
+			setSelected(PARTS_BYTESWAP[0], false);
+			break;
+		case PART1_CLEAR:
+			part_path[1][0] = '\0';
+			part_size[1][0] = '\0';
+			setSelected(PARTS_PRESENT[1], false);
+			setSelected(PARTS_READONLY[1], false);
+			setSelected(PARTS_BYTESWAP[1], false);
+			break;
+		case PART2_CLEAR:
+			part_path[2][0] = '\0';
+			part_size[2][0] = '\0';
+			setSelected(PARTS_PRESENT[2], false);
+			setSelected(PARTS_READONLY[2], false);
+			setSelected(PARTS_BYTESWAP[2], false);
+			break;
+		case PART3_CLEAR:
+			part_path[3][0] = '\0';
+			part_size[3][0] = '\0';
+			setSelected(PARTS_PRESENT[3], false);
+			setSelected(PARTS_READONLY[3], false);
+			setSelected(PARTS_BYTESWAP[3], false);
+			break;
+
 		case PART0_GENERATE:
 			init_create_disk_image(0);
 			break;
@@ -358,6 +254,7 @@ int DlgPartition::processDialogMain(void)
 
 		case APPLY:
 			confirm();
+			/* fall through */
 		case CANCEL:
 			retval = Dialog::GUI_CLOSE;
 			break;
