@@ -19,12 +19,6 @@ enum {
   FILEFILTER_MEMORYFILE
 };
 
-#if defined(__BEOS__) || defined(OS_solaris) || defined(OS_mingw)
-#include <dirent.h>
-extern int alphasort(const void *d1, const void *d2);
-extern int scandir(const char *dirname, struct dirent ***namelist, int(*select) (const struct dirent *), int (*dcomp) (const void *, const void *));
-#endif  /* __BEOS__ */
-
 extern void File_CleanFileName(char *pszFileName);
 extern void File_AddSlashToEndFileName(char *pszFileName);
 extern bool File_DoesFileExtensionMatch(char *pszFileName, char *pszExtension);
