@@ -24,7 +24,7 @@
 #include "sysdeps.h"
 #include "hardware.h"
 #include "cpu_emulation.h"
-#include "memory.h"
+#include "memory-uae.h"
 #include "ikbd.h"
 
 #define DEBUG 0
