@@ -22,7 +22,7 @@
 #define LOGO_H 1
 
 #ifndef LOGO_FILENAME
-#define LOGO_FILENAME	"logo.png"
+#define LOGO_FILENAME	"logo.bmp"
 #endif
 
 class HostSurface;
@@ -33,13 +33,16 @@ class Logo {
 	private:
 		SDL_Surface *logo_surf;
 		HostSurface *surface;
+		int opacity;
+		
+		void load(const char *filename);
 
 	public:
 		Logo(const char *filename);
 		virtual ~Logo();
 
-		void load(const char *filename);
 		HostSurface *getSurface(void);
+		void alphaBlend(bool init);
 };
 
 #endif /* LOGO_H */
