@@ -110,6 +110,8 @@ typedef struct {
   bool redirect_CON;
   long cookie_mch;
   long cookie_akp;
+  char			tos_path[512];
+  char			emutos_path[512];
 } bx_tos_options_t;
 
 // Video output options
@@ -134,6 +136,7 @@ typedef struct {
   bool jit;
   bool jitfpu;
   bool jitinline;
+  bool jitdebug;
   uint32 jitcachesize;
   uint32 jitlazyflush;
   char jitblacklist[512];
@@ -143,7 +146,7 @@ typedef struct {
 typedef struct {
   bool enabled;
   bool filtered;
-  char library[512];
+  char library[1024];
 } bx_opengl_options_t;
 
 // Ethernet options
@@ -169,6 +172,7 @@ typedef struct {
 	char type[32];		/* MIDI output type */
 	char file[512];	/* /path/to/output */
 	char sequencer[256];	/* /dev/sequencer */
+	bool enabled;
 } bx_midi_options_t;
 
 // NfCdrom options
@@ -177,12 +181,10 @@ typedef struct {
 } bx_nfcdrom_options_t;
 
 // Misc CPU emulation options
-#ifdef ENABLE_EPSLIMITER
 typedef struct {
 	bool	eps_enabled;	/* Exception per second limiter enabled ? */
 	int32	eps_max;		/* Maximum tolerated eps before shutdown */
 } bx_cpu_options_t;
-#endif
 
 // Autozoom options
 typedef struct {
@@ -196,14 +198,15 @@ typedef struct {
 // NfOSMesa options
 typedef struct {
 	uint32 channel_size;    /* If using libOSMesa[16|32], size of channel */
-	char libgl[256];		/* Pathname to libGL */
-	char libosmesa[256];	/* Pathname to libOSMesa */
+	char libgl[1024];		/* Pathname to libGL */
+	char libosmesa[1024];	/* Pathname to libOSMesa */
 } bx_nfosmesa_options_t;
 
 // NatFeats options
 typedef struct {
 	char cdrom_driver[256];	/* CD-ROM driver */
 	char vdi_driver[256];	/* VDI driver */
+	bool hostexec_enabled;
 } bx_natfeat_options_t;
 
 // NFvdi options
@@ -216,11 +219,13 @@ typedef struct {
 	char type[256];
 	char file[256];
 	char parport[256];
+	bool enabled;
 } bx_parallel_options_t;
 
 // Serial port options
 typedef struct {
 	char serport[256];
+	bool enabled;
 } bx_serial_options_t;
 
 // Keyboard and mouse
@@ -239,11 +244,13 @@ typedef struct {
 typedef struct {
 	bx_hotkey	setup;
 	bx_hotkey	quit;
-	bx_hotkey	reboot;
+	bx_hotkey	warmreboot;
+	bx_hotkey	coldreboot;
 	bx_hotkey	debug;
 	bx_hotkey	ungrab;
 	bx_hotkey	fullscreen;
 	bx_hotkey	screenshot;
+	bx_hotkey	sound;
 } bx_hotkeys_t;
 
 // Audio
@@ -252,6 +259,7 @@ typedef struct {
 	uint32 chans;
 	uint32 bits;
 	uint32 samples;
+	bool enabled;
 } bx_audio_options_t;
 
 // Joysticks
@@ -284,9 +292,7 @@ typedef struct {
   bx_midi_options_t		midi;
   bx_ikbd_options_t		ikbd;
   bx_nfcdrom_options_t	nfcdroms[ CD_MAX_DRIVES ];
-#ifdef ENABLE_EPSLIMITER
   bx_cpu_options_t  cpu;
-#endif
   bx_autozoom_options_t	autozoom;
   bx_nfosmesa_options_t	osmesa;
   bx_parallel_options_t parallel;
@@ -295,51 +301,46 @@ typedef struct {
   bx_nfvdi_options_t	nfvdi;
   bx_audio_options_t	audio;
   bx_joysticks_options_t	joysticks;
-  char			tos_path[512];
-  char			emutos_path[512];
   char			bootstrap_path[512];
   char			bootstrap_args[512];
   char			bootdrive;
   uint32		fastram;
+  unsigned long fixed_memory_offset; // FIXME: should be uintptr
   bool			gmtime;
   bx_hotkeys_t		hotkeys;
   bool			newHardDriveSupport;
+  char			snapshot_dir[512];
 } bx_options_t;
 
 extern bx_options_t bx_options;
 
 
 extern uint32 FastRAMSize;	// Size of Fast-RAM
+#if FIXED_ADDRESSING
+extern uintptr fixed_memory_offset;	// Virtual address of atari memory
+#endif
 
 extern char *program_name;
-extern char rom_path[512];
-extern char emutos_path[512];
-extern bool startupGUI;
 extern bool boot_emutos;
 extern bool boot_lilo;
 extern bool halt_on_reboot;
 
-void usage(int);
-extern bool check_cfg();
 extern bool decode_switches(int, char **);
 extern char *getConfFilename(const char *file, char *buffer, unsigned int bufsize);
 extern char *getDataFilename(const char *file, char *buffer, unsigned int bufsize);
 char *addFilename(char *buffer, const char *file, unsigned int bufsize);
-
-// following functions implemented in parameters_[unix|linux|cygwin].cpp
-char *getHomeFolder(char *buffer, unsigned int bufsize);
-char *getConfFolder(char *buffer, unsigned int bufsize);
-char *getDataFolder(char *buffer, unsigned int bufsize);
+bool setConfigValue(const char *section_name, const char *key, const char *value);
+void listConfigValues(bool type);
 
 char *keysymToString(char *buffer, const bx_hotkey *keysym);
 bool stringToKeysym(bx_hotkey *keysym, const char *string);
+char **split_pathlist(const char *pathlist);
 
 extern const char *getConfigFile();
+void setConfigFile(const char *filename);
 extern bool loadSettings(const char *);
 extern bool saveSettings(const char *);
 
-#endif
+extern const struct Config_Section *getConfigSections(void);
 
-/*
-vim:ts=4:sw=4:
-*/
+#endif
