@@ -21,20 +21,21 @@
 #include "sysdeps.h"
 
 #include "SDL_compat.h"
-#ifdef HAVE_SDL_IMAGE
-#include <SDL_image.h>
-#endif
 
 #include "dirty_rects.h"
 #include "host_surface.h"
 #include "logo.h"
 #include "hostscreen.h"
 #include "host.h"
+
+#define DEBUG 0
+#include "debug.h"
+
  
 /*--- Constructor/destructor ---*/
 
 Logo::Logo(const char *filename)
-	: logo_surf(NULL), surface(NULL)
+	: logo_surf(NULL), surface(NULL), opacity(100)
 {
 	load(filename);
 }
@@ -58,19 +59,9 @@ void Logo::load(const char *filename)
 		surface = NULL;
 	}
 
-	SDL_RWops *rwops = SDL_RWFromFile(filename, "rb");
-	if (!rwops) {
-		return;
-	}
-
-#ifdef HAVE_SDL_IMAGE
-	logo_surf = IMG_Load_RW(rwops, 0);
-#else
-	logo_surf = SDL_LoadBMP_RW(rwops, 0);
-#endif
-	SDL_FreeRW(rwops);
-
+	logo_surf = mySDL_LoadBMP_RW(SDL_RWFromFile(filename, "rb"), 1);
 	if (!logo_surf) {
+		panicbug("Can not load logo from file %s", filename); 
 		return;
 	}
 
@@ -122,6 +113,27 @@ HostSurface *Logo::getSurface(void)
 	return surface;
 }
 
+void Logo::alphaBlend(bool init)
+{
+	if (init) {
+		opacity = 100;
+	}
+	else {
+		if (bx_options.opengl.enabled && opacity > 0) {
+			if (surface != NULL) {
+				surface->setParam(HostSurface::SURF_ALPHA, opacity);
+				surface->setDirtyRect(0, 0, surface->getWidth(), surface->getHeight());
+				host->video->drawSurfaceToScreen(surface);
+			}
+
+			opacity -= 5 * bx_options.video.refresh;
+			if (opacity <= 0 && getVIDEL() != NULL && getVIDEL()->getSurface() != NULL && surface != NULL) {
+				getVIDEL()->getSurface()->setDirtyRect(0, 0, surface->getWidth(), surface->getHeight());
+			}
+		}
+	}
+}
+
 /*
 vim:ts=4:sw=4:
 */
