@@ -33,6 +33,8 @@
 #include <cstring>
 #include "SDL_compat.h"
 
+#define ZMAGXSND_BUFSIZE 4096
+
 extern "C" {
 	static SDL_AudioStatus playing;
 	static SDL_AudioCVT	cvt;
@@ -53,9 +55,13 @@ extern "C" {
 		if (cvt.needed) {
 			/* Convert Atari audio to host audio */
 			par->len = (uint32 ) (len / cvt.len_ratio);
-
+			if (par->len > ZMAGXSND_BUFSIZE) {
+				par->len = ZMAGXSND_BUFSIZE;
+			}
+			uint32 bufSize = par->len * cvt.len_mult;
+			
 			/* Current buffer too small ? */
-			if (cvt_buf_len<par->len) {
+			if (cvt_buf_len<bufSize) {
 				if (cvt.buf) {
 					free(cvt.buf);
 					cvt.buf=NULL;
@@ -64,17 +70,21 @@ extern "C" {
 
 			/* Allocate needed buffer */
 			if (cvt.buf==NULL) {
-				cvt.buf=(uint8 *)malloc(par->len);
-				cvt_buf_len = par->len;
+				cvt.buf=(uint8 *)malloc(bufSize);
+				cvt_buf_len = bufSize;
 			}
 
 			memcpy(cvt.buf, buffer, par->len);
 			cvt.len = par->len;
 			SDL_ConvertAudio(&cvt);
-			SDL_MixAudio(stream, cvt.buf, len, par->volume);
+			SDL_MixAudio(stream, cvt.buf, cvt.len_cvt, par->volume);
 		} else {
 			par->len = len;
-			SDL_MixAudio(stream, buffer, len, par->volume);
+			if (par->len > ZMAGXSND_BUFSIZE) {
+				par->len = ZMAGXSND_BUFSIZE;
+			}
+
+			SDL_MixAudio(stream, buffer, par->len, par->volume);
 		}
 
 		TriggerInt5();		// Audio is at interrupt level 5
