@@ -99,7 +99,7 @@
 
 #include "sysdeps.h"
 #include <cstdio>
-#include "memory.h"
+#include "memory-uae.h"
 #include "readcpu.h"
 #include "newcpu.h"
 #include "main.h"
@@ -141,6 +141,24 @@ fpu_t fpu;
 #include "fpu/exceptions.cpp"
 #include "fpu/rounding.cpp"
 
+#if defined(USE_LONG_DOUBLE) || defined(USE_QUAD_DOUBLE)
+#define LD(x) x ## L
+#ifdef HAVE_POWL
+#define POWL(x, y) powl(x, y)
+#else
+#define POWL(x, y) pow(x, y)
+#endif
+#ifdef HAVE_LOG10L
+#define LOG10L(x) log10l(x)
+#else
+#define LOG10L(x) log10(x)
+#endif
+#else
+#define LD(x) x
+#define POWL(x, y) pow(x, y)
+#define LOG10L(x) log10(x)
+#endif
+
 /* -------------------------------------------------------------------------- */
 /* --- Debugging                                                          --- */
 /* -------------------------------------------------------------------------- */
@@ -293,15 +311,12 @@ PRIVATE inline fpu_register FFPU make_extended(uae_u32 wrd1, uae_u32 wrd2, uae_u
 #if defined(USE_QUAD_DOUBLE)
 	// is it NaN?
 	if ((wrd1 & 0x7fff0000) == 0x7fff0000 && ((wrd2 & 0x7fffffff) != 0 || wrd3 != 0)) {
-		make_nan(result);
+		make_nan(result, (wrd1 & 0x80000000) != 0);
 		return result;
 	}
 	// is it inf?
 	if ((wrd1 & 0x7ffff000) == 0x7fff0000 && (wrd2 & 0x7fffffff) == 0 && wrd3 == 0) {
-		if ((wrd1 & 0x80000000) == 0)
-			make_inf_positive(result);
-		else
-			make_inf_negative(result);
+		make_inf(result, (wrd1 & 0x80000000) != 0);
 		return result;
 	}
 	fp_declare_init_shape(srp, extended);
@@ -369,24 +384,18 @@ PRIVATE inline void FFPU make_extended_no_normalize(
 {
 	// is it zero?
 	if ((wrd1 & 0x7fff0000) == 0 && wrd2 == 0 && wrd3 == 0) {
-		if (wrd1 & 0x80000000)
-			make_zero_negative(result);
-		else
-			make_zero_positive(result);
+		make_zero(result, (wrd1 & 0x80000000) != 0);
 		return;
 	}
 	// is it NaN?
 	if ((wrd1 & 0x7fff0000) == 0x7fff0000 && ((wrd2 & 0x7fffffff) != 0 || wrd3 != 0)) {
-		make_nan(result);
+		make_nan(result, (wrd1 & 0x80000000) != 0);
 		return;
 	}
 #if defined(USE_QUAD_DOUBLE)
 	// is it inf?
 	if ((wrd1 & 0x7ffff000) == 0x7fff0000 && (wrd2 & 0x7fffffff) == 0 && wrd3 == 0) {
-		if ((wrd1 & 0x80000000) == 0)
-			make_inf_positive(result);
-		else
-			make_inf_negative(result);
+		make_inf(result, (wrd1 & 0x80000000) != 0);
 		return;
 	}
 	fp_declare_init_shape(srp, extended);
@@ -513,41 +522,88 @@ PRIVATE inline void FFPU extract_double(fpu_register const & src,
 // to_pack
 PRIVATE inline fpu_register FFPU make_packed(uae_u32 wrd1, uae_u32 wrd2, uae_u32 wrd3)
 {
-	fpu_double d;
-	char *cp;
-	char str[100];
+	fpu_register d;
+	bool sm = (wrd1 & 0x80000000) != 0;
+	bool se = (wrd1 & 0x40000000) != 0;
+	int exp = (wrd1 & 0x7fff0000) >> 16;
+	unsigned int dig;
+	fpu_register pwr;
+	
+	if (exp == 0x7fff)
+	{
+		if ((wrd2 & 0x7fffffff) == 0 && wrd3 == 0)
+		{
+			make_inf(d, sm);
+		} else
+		{
+			make_nan(d, sm);
+		}
+		return d;
+	}
+	dig = wrd1 & 0x0000000f;
+	if (dig == 0 && wrd2 == 0 && wrd3 == 0)
+	{
+		make_zero(d, sm);
+		return d;
+	}
 
-	cp = str;
-	if (wrd1 & 0x80000000)
-		*cp++ = '-';
-	*cp++ = (char)((wrd1 & 0xf) + '0');
-	*cp++ = '.';
-	*cp++ = (char)(((wrd2 >> 28) & 0xf) + '0');
-	*cp++ = (char)(((wrd2 >> 24) & 0xf) + '0');
-	*cp++ = (char)(((wrd2 >> 20) & 0xf) + '0');
-	*cp++ = (char)(((wrd2 >> 16) & 0xf) + '0');
-	*cp++ = (char)(((wrd2 >> 12) & 0xf) + '0');
-	*cp++ = (char)(((wrd2 >> 8) & 0xf) + '0');
-	*cp++ = (char)(((wrd2 >> 4) & 0xf) + '0');
-	*cp++ = (char)(((wrd2 >> 0) & 0xf) + '0');
-	*cp++ = (char)(((wrd3 >> 28) & 0xf) + '0');
-	*cp++ = (char)(((wrd3 >> 24) & 0xf) + '0');
-	*cp++ = (char)(((wrd3 >> 20) & 0xf) + '0');
-	*cp++ = (char)(((wrd3 >> 16) & 0xf) + '0');
-	*cp++ = (char)(((wrd3 >> 12) & 0xf) + '0');
-	*cp++ = (char)(((wrd3 >> 8) & 0xf) + '0');
-	*cp++ = (char)(((wrd3 >> 4) & 0xf) + '0');
-	*cp++ = (char)(((wrd3 >> 0) & 0xf) + '0');
-	*cp++ = 'E';
-	if (wrd1 & 0x40000000)
-		*cp++ = '-';
-	*cp++ = (char)(((wrd1 >> 24) & 0xf) + '0');
-	*cp++ = (char)(((wrd1 >> 20) & 0xf) + '0');
-	*cp++ = (char)(((wrd1 >> 16) & 0xf) + '0');
-	*cp = 0;
-	sscanf(str, "%le", &d);
+	/*
+	 * Convert the bcd exponent to binary by successive adds and
+	 * muls. Set the sign according to SE. Subtract 16 to compensate
+	 * for the mantissa which is to be interpreted as 17 integer
+	 * digits, rather than 1 integer and 16 fraction digits.
+	 * Note: this operation can never overflow.
+	 */
+	exp = ((wrd1 >> 24) & 0xf);
+	exp = exp * 10 + ((wrd1 >> 20) & 0xf);
+	exp = exp * 10 + ((wrd1 >> 16) & 0xf);
+	if (se)
+		exp = -exp;
+	/* sub to compensate for shift of mant */
+	exp = exp - 16;
+	
+	/*
+	 * Convert the bcd mantissa to binary by successive
+	 * adds and muls. Set the sign according to SM.
+	 * The mantissa digits will be converted with the decimal point
+	 * assumed following the least-significant digit.
+	 * Note: this operation can never overflow.
+	 */
+	d = wrd1 & 0xf;
+	d = (d * LD(10.0)) + ((wrd2 >> 28) & 0xf);
+	d = (d * LD(10.0)) + ((wrd2 >> 24) & 0xf);
+	d = (d * LD(10.0)) + ((wrd2 >> 20) & 0xf);
+	d = (d * LD(10.0)) + ((wrd2 >> 16) & 0xf);
+	d = (d * LD(10.0)) + ((wrd2 >> 12) & 0xf);
+	d = (d * LD(10.0)) + ((wrd2 >>  8) & 0xf);
+	d = (d * LD(10.0)) + ((wrd2 >>  4) & 0xf);
+	d = (d * LD(10.0)) + ((wrd2      ) & 0xf);
+	d = (d * LD(10.0)) + ((wrd3 >> 28) & 0xf);
+	d = (d * LD(10.0)) + ((wrd3 >> 24) & 0xf);
+	d = (d * LD(10.0)) + ((wrd3 >> 20) & 0xf);
+	d = (d * LD(10.0)) + ((wrd3 >> 16) & 0xf);
+	d = (d * LD(10.0)) + ((wrd3 >> 12) & 0xf);
+	d = (d * LD(10.0)) + ((wrd3 >>  8) & 0xf);
+	d = (d * LD(10.0)) + ((wrd3 >>  4) & 0xf);
+	d = (d * LD(10.0)) + ((wrd3      ) & 0xf);
 
-	fpu_debug(("make_packed str = %s\n",str));
+	/* Check the sign of the mant and make the value in fp0 the same sign. */
+	if (sm)
+		d = -d;
+	
+	/*
+	 * Calculate power-of-ten factor from exponent.
+	 */
+	if (exp < 0)
+	{
+		exp = -exp;
+		pwr = POWL(LD(10.0), exp);
+		d = d / pwr;
+	} else
+	{
+		pwr = POWL(LD(10.0), exp);
+		d = d * pwr;
+	}
 
 	fpu_debug(("make_packed(%X,%X,%X) = %.04f\n",wrd1,wrd2,wrd3,(double)d));
 	return d;
@@ -556,56 +612,88 @@ PRIVATE inline fpu_register FFPU make_packed(uae_u32 wrd1, uae_u32 wrd2, uae_u32
 // from_pack
 PRIVATE inline void FFPU extract_packed(fpu_register const & src, uae_u32 * wrd1, uae_u32 * wrd2, uae_u32 * wrd3)
 {
-	int i;
-	int t;
-	char *cp;
-	char str[100];
-
-#ifdef _WIN32
-	sprintf(str, "%.16e", (double)src);
-#else
-	sprintf(str, "%.16Le", (long double)src);
-#endif
-
-	fpu_debug(("extract_packed(%.04f,%s)\n",(double)src,str));
-
-	cp = str;
+	fpu_register pwr;
+	int exp;
+	fpu_register d;
+	bool sm, se;
+	int dig;
+	
 	*wrd1 = *wrd2 = *wrd3 = 0;
-	if (*cp == '-') {
-		cp++;
-		*wrd1 = 0x80000000;
-	}
-	if (*cp == '+')
-		cp++;
-	*wrd1 |= (*cp++ - '0');
-	if (*cp == '.')
-		cp++;
-	for (i = 0; i < 8; i++) {
-		*wrd2 <<= 4;
-		if (*cp >= '0' && *cp <= '9')
-		*wrd2 |= *cp++ - '0';
-	}
-	for (i = 0; i < 8; i++) {
-		*wrd3 <<= 4;
-		if (*cp >= '0' && *cp <= '9')
-		*wrd3 |= *cp++ - '0';
-	}
-	if (*cp == 'e' || *cp == 'E') {
-		cp++;
-		if (*cp == '-') {
-			cp++;
-			*wrd1 |= 0x40000000;
-		}
-		if (*cp == '+')
-			cp++;
-		t = 0;
-		for (i = 0; i < 3; i++) {
-			if (*cp >= '0' && *cp <= '9')
-				t = (t << 4) | (*cp++ - '0');
-		}
-		*wrd1 |= t << 16;
+	
+	d = src;
+	sm = false;
+	if (isneg(src))
+	{
+		d = -d;
+		sm = true;
 	}
 
+	if (isnan(src))
+	{
+		*wrd1 = sm ? 0xffff0000 : 0x7fff0000;
+		*wrd2 = 0xffffffff;
+		*wrd3 = 0xffffffff;
+		return;
+	}
+	if (isinf(src))
+	{
+		*wrd1 = sm ? 0xffff0000 : 0x7fff0000;
+		*wrd2 = *wrd3 = 0;
+		return;
+	}
+	if (iszero(src))
+	{
+		*wrd1 = sm ? 0x80000000 : 0x00000000;
+		*wrd2 = *wrd3 = 0;
+		return;
+	}
+	sm = false;
+	if (isneg(src))
+	{
+		d = -d;
+		sm = true;
+	}
+	exp = (int)floor(LOG10L(d));
+	se = false;
+	if (exp < 0)
+	{
+		exp = -exp;
+		se = true;
+		pwr = POWL(LD(10.0), exp);
+		d = d * pwr;
+	} else
+	{
+		pwr = POWL(LD(10.0), exp);
+		d = d / pwr;
+	}
+	dig = (int)d; d = LD(10) * (d - dig); *wrd1 |= dig;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd2 |= dig << 28;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd2 |= dig << 24;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd2 |= dig << 20;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd2 |= dig << 16;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd2 |= dig << 12;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd2 |= dig <<  8;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd2 |= dig <<  4;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd2 |= dig;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd3 |= dig << 28;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd3 |= dig << 24;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd3 |= dig << 20;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd3 |= dig << 16;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd3 |= dig << 12;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd3 |= dig <<  8;
+	dig = (int)d; d = LD(10) * (d - dig); *wrd3 |= dig <<  4;
+	dig = (int)d;                         *wrd3 |= dig;
+	
+	dig = (exp / 100) % 10;
+	*wrd1 |= dig << 24;
+	dig = (exp / 10) % 10;
+	*wrd1 |= dig << 20;
+	dig = (exp) % 10;
+	*wrd1 |= dig << 16;
+	if (sm)
+		*wrd1 |= 0x80000000;
+	if (se)
+		*wrd1 |= 0x40000000;
 	fpu_debug(("extract_packed(%.04f) = %X,%X,%X\n",(double)src,*wrd1,*wrd2,*wrd3));
 }
 
@@ -764,10 +852,10 @@ PRIVATE inline int FFPU get_fp_value (uae_u32 opcode, uae_u16 extra, fpu_registe
 }
 
 /* Convert the FP value to integer according to the current m68k rounding mode */
-PRIVATE inline uae_s32 FFPU toint(fpu_register const & src)
+PRIVATE inline fpu_register FFPU fp_doround(fpu_register const & src)
 {
 	fpu_register result;
-	switch (get_fpcr() & 0x30) {
+	switch (get_fpcr() & FPCR_ROUNDING_MODE) {
 	case FPCR_ROUND_ZERO:
 		result = fp_round_to_zero(src);
 		break;
@@ -784,7 +872,12 @@ PRIVATE inline uae_s32 FFPU toint(fpu_register const & src)
 		result = src; /* should never be reached */
 		break;
 	}
-	return (uae_s32)result;
+	return result;
+}
+
+PRIVATE inline uae_s32 FFPU toint(fpu_register const & src)
+{
+	return (uae_s32)fp_doround(src);
 }
 
 PRIVATE inline int FFPU put_fp_value (uae_u32 opcode, uae_u16 extra, fpu_register const & value)
@@ -1294,8 +1387,7 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 		if ((opcode & 0x38) == 0) {
 			if (extra & 0x2000) { // dr bit
 				if (extra & 0x1000) {
-					// according to the manual, the msb bits are always zero.
-					m68k_dreg (regs, opcode & 7) = get_fpcr() & 0xFFFF;
+					m68k_dreg (regs, opcode & 7) = get_fpcr();
 					fpu_debug(("FMOVEM FPU fpcr (%X) -> D%d\n", get_fpcr(), opcode & 7));
 				}
 				if (extra & 0x0800) {
@@ -1321,13 +1413,11 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 					fpu_debug(("FMOVEM D%d (%X) -> FPU instruction_address\n", opcode & 7, FPU instruction_address));
 				}
 			}
-//		} else if ((opcode & 0x38) == 1) {
 		}
 		else if ((opcode & 0x38) == 8) { 
 			if (extra & 0x2000) { // dr bit
 				if (extra & 0x1000) {
-					// according to the manual, the msb bits are always zero.
-					m68k_areg (regs, opcode & 7) = get_fpcr() & 0xFFFF;
+					m68k_areg (regs, opcode & 7) = get_fpcr();
 					fpu_debug(("FMOVEM FPU fpcr (%X) -> A%d\n", get_fpcr(), opcode & 7));
 				}
 				if (extra & 0x0800) {
@@ -1390,8 +1480,7 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			}
 			ad -= incr;
 			if (extra & 0x1000) {
-				// according to the manual, the msb bits are always zero.
-				put_long (ad, get_fpcr() & 0xFFFF);
+				put_long (ad, get_fpcr());
 				fpu_debug(("FMOVEM FPU fpcr (%X) -> mem %X\n", get_fpcr(), ad ));
 				ad += 4;
 			}
@@ -1606,27 +1695,27 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			switch (extra & 0x7f) {
 			case 0x00:
 				// FPU registers[reg] = 4.0 * atan(1.0);
-				FPU registers[reg] = 3.1415926535897932384626433832795;
+				FPU registers[reg] = LD(3.1415926535897932384626433832795029);
 				fpu_debug(("FP const: Pi\n"));
 				break;
 			case 0x0b:
 				// FPU registers[reg] = log10 (2.0);
-				FPU registers[reg] = 0.30102999566398119521373889472449;
+				FPU registers[reg] = LD(0.30102999566398119521); // 0.3010299956639811952137388947244930L
 				fpu_debug(("FP const: Log 10 (2)\n"));
 				break;
 			case 0x0c:
 				// FPU registers[reg] = exp (1.0);
-				FPU registers[reg] = 2.7182818284590452353602874713527;
+				FPU registers[reg] = LD(2.7182818284590452353); // 2.7182818284590452353602874713526625L
 				fpu_debug(("FP const: e\n"));
 				break;
 			case 0x0d:
 				// FPU registers[reg] = log (exp (1.0)) / log (2.0);
-				FPU registers[reg] = 1.4426950408889634073599246810019;
+				FPU registers[reg] = LD(1.4426950408889634073599246810019);
 				fpu_debug(("FP const: Log 2 (e)\n"));
 				break;
 			case 0x0e:
 				// FPU registers[reg] = log (exp (1.0)) / log (10.0);
-				FPU registers[reg] = 0.43429448190325182765112891891661;
+				FPU registers[reg] = LD(0.4342944819032518276511289189166051);
 				fpu_debug(("FP const: Log 10 (e)\n"));
 				break;
 			case 0x0f:
@@ -1635,73 +1724,79 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 				break;
 			case 0x30:
 				// FPU registers[reg] = log (2.0);
-				FPU registers[reg] = 0.69314718055994530941723212145818;
+				FPU registers[reg] = LD(0.6931471805599453094172321214581766);
 				fpu_debug(("FP const: ln(2)\n"));
 				break;
 			case 0x31:
 				// FPU registers[reg] = log (10.0);
-				FPU registers[reg] = 2.3025850929940456840179914546844;
+				FPU registers[reg] = LD(2.3025850929940456840179914546843642);
 				fpu_debug(("FP const: ln(10)\n"));
 				break;
 			case 0x32:
-				// ??
-				FPU registers[reg] = 1.0e0;
+				FPU registers[reg] = LD(1.0e0);
 				fpu_debug(("FP const: 1.0e0\n"));
 				break;
 			case 0x33:
-				FPU registers[reg] = 1.0e1;
+				FPU registers[reg] = LD(1.0e1);
 				fpu_debug(("FP const: 1.0e1\n"));
 				break;
 			case 0x34:
-				FPU registers[reg] = 1.0e2;
+				FPU registers[reg] = LD(1.0e2);
 				fpu_debug(("FP const: 1.0e2\n"));
 				break;
 			case 0x35:
-				FPU registers[reg] = 1.0e4;
+				FPU registers[reg] = LD(1.0e4);
 				fpu_debug(("FP const: 1.0e4\n"));
 				break;
 			case 0x36:
-				FPU registers[reg] = 1.0e8;
+				FPU registers[reg] = LD(1.0e8);
 				fpu_debug(("FP const: 1.0e8\n"));
 				break;
 			case 0x37:
-				FPU registers[reg] = 1.0e16;
+				FPU registers[reg] = LD(1.0e16);
 				fpu_debug(("FP const: 1.0e16\n"));
 				break;
 			case 0x38:
-				FPU registers[reg] = 1.0e32;
+				FPU registers[reg] = LD(1.0e32);
 				fpu_debug(("FP const: 1.0e32\n"));
 				break;
 			case 0x39:
-				FPU registers[reg] = 1.0e64;
+				FPU registers[reg] = LD(1.0e64);
 				fpu_debug(("FP const: 1.0e64\n"));
 				break;
 			case 0x3a:
-				FPU registers[reg] = 1.0e128;
+				FPU registers[reg] = LD(1.0e128);
 				fpu_debug(("FP const: 1.0e128\n"));
 				break;
 			case 0x3b:
-				FPU registers[reg] = 1.0e256;
+				FPU registers[reg] = LD(1.0e256);
 				fpu_debug(("FP const: 1.0e256\n"));
 				break;
 #if defined(USE_LONG_DOUBLE) || defined(USE_QUAD_DOUBLE)
 			case 0x3c:
-				FPU registers[reg] = 1.0e512L;
+				FPU registers[reg] = LD(1.0e512);
 				fpu_debug(("FP const: 1.0e512\n"));
 				break;
 			case 0x3d:
-				FPU registers[reg] = 1.0e1024L;
+				FPU registers[reg] = LD(1.0e1024);
 				fpu_debug(("FP const: 1.0e1024\n"));
 				break;
 			case 0x3e:
-				FPU registers[reg] = 1.0e2048L;
+				FPU registers[reg] = LD(1.0e2048);
 				fpu_debug(("FP const: 1.0e2048\n"));
 				break;
 			case 0x3f:
-				FPU registers[reg] = 1.0e4096L;
+				FPU registers[reg] = LD(1.0e4096);
 				fpu_debug(("FP const: 1.0e4096\n"));
-#endif
 				break;
+#else
+			case 0x3c:
+			case 0x3d:
+			case 0x3e:
+			case 0x3f:
+				make_inf(FPU registers[reg], false);
+				break;
+#endif
 			default:
 				m68k_setpc (m68k_getpc () - 4);
 				op_illg (opcode);
@@ -1808,25 +1903,13 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 				else if (fl_dest.nan || fl_source.nan || 
 						 (fl_dest.zero && fl_source.infinity) || 
 						 (fl_dest.infinity && fl_source.zero) ) {
-					make_nan( FPU registers[reg] );
+					make_nan( FPU registers[reg], fl_dest.negative );
 				}
 				else if (fl_dest.zero || fl_source.zero ) {
-					if ( (fl_dest.negative && !fl_source.negative) ||
-						(!fl_dest.negative && fl_source.negative) )  {
-						make_zero_negative(FPU registers[reg]);
-					}
-					else {
-						make_zero_positive(FPU registers[reg]);
-					}
+					make_zero(FPU registers[reg], fl_dest.negative != fl_source.negative);
 				}
 				else {
-					if ( (fl_dest.negative && !fl_source.negative) ||
-						(!fl_dest.negative && fl_source.negative) )  {
-						make_inf_negative(FPU registers[reg]);
-					}
-					else {
-						make_inf_positive(FPU registers[reg]);
-					}
+					make_inf(FPU registers[reg], fl_dest.negative != fl_source.negative);
 				}
 				make_fpsr(FPU registers[reg]);
 				break;
@@ -1847,43 +1930,68 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x01:		/* FINT */
+			/*
+			 * FIXME: in round-to-nearest, x87
+			 * uses round-to-odd, but m68k round-to-even rule
+			 */
 			fpu_debug(("FINT %.04f\n",(double)src));
-			FPU registers[reg] = toint(src);
+			if (isinf(src))
+				FPU registers[reg] = src;
+			else
+				FPU registers[reg] = fp_doround(src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x02:		/* FSINH */
 			fpu_debug(("FSINH %.04f\n",(double)src));
-			FPU registers[reg] = fp_sinh (src);
+			if (isinf(src))
+				FPU registers[reg] = src;
+			else
+				FPU registers[reg] = fp_sinh (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x03:		/* FINTRZ */
 			fpu_debug(("FINTRZ %.04f\n",(double)src));
-			FPU registers[reg] = fp_round_to_zero(src);
+			if (isinf(src))
+				FPU registers[reg] = src;
+			else
+				FPU registers[reg] = fp_round_to_zero(src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x04:		/* FSQRT */
 			fpu_debug(("FSQRT %.04f\n",(double)src));
-			FPU registers[reg] = fp_sqrt (src);
+			if (isinf(src) && !isneg(src))
+				FPU registers[reg] = src;
+			else
+				FPU registers[reg] = fp_sqrt (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x06:		/* FLOGNP1 */
 			fpu_debug(("FLOGNP1 %.04f\n",(double)src));
-			FPU registers[reg] = fp_log (src + 1.0);
+			if (isinf(src) && !isneg(src))
+				make_inf(FPU registers[reg], false);
+			else
+				FPU registers[reg] = fp_log1p (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x08:		/* FETOXM1 */
 			fpu_debug(("FETOXM1 %.04f\n",(double)src));
-			FPU registers[reg] = fp_exp (src) - 1.0;
+			FPU registers[reg] = fp_expm1 (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x09:		/* FTANH */
 			fpu_debug(("FTANH %.04f\n",(double)src));
-			FPU registers[reg] = fp_tanh (src);
+			if (isinf(src))
+				FPU registers[reg] = isneg(src) ? LD(-1.0) : LD(1.0);
+			else
+				FPU registers[reg] = fp_tanh (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x0a:		/* FATAN */
 			fpu_debug(("FATAN %.04f\n",(double)src));
-			FPU registers[reg] = fp_atan (src);
+			if (isinf(src))
+				FPU registers[reg] = isneg (src) ? LD(-1.570796326794896619231321691639751442) : LD(1.570796326794896619231321691639751442);
+			else
+				FPU registers[reg] = fp_atan (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x0c:		/* FASIN */
@@ -1908,32 +2016,65 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			break;
 		case 0x10:		/* FETOX */
 			fpu_debug(("FETOX %.04f\n",(double)src));
-			FPU registers[reg] = fp_exp (src);
+			if (isinf(src))
+			{
+				make_zero(FPU registers[reg], isneg(src));
+			} else
+			{
+				FPU registers[reg] = fp_exp (src);
+			}
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x11:		/* FTWOTOX */
 			fpu_debug(("FTWOTOX %.04f\n",(double)src));
-			FPU registers[reg] = fp_pow(2.0, src);
+			if (isinf(src))
+			{
+				if (isneg(src))
+					make_zero(FPU registers[reg], false);
+				else
+					make_inf(FPU registers[reg], true);
+			} else
+			{
+				FPU registers[reg] = fp_pow2(src);
+			}
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x12:		/* FTENTOX */
 			fpu_debug(("FTENTOX %.04f\n",(double)src));
-			FPU registers[reg] = fp_pow(10.0, src);
+			if (isinf(src))
+			{
+				if (isneg(src))
+					make_zero(FPU registers[reg], false);
+				else
+					make_inf(FPU registers[reg], true);
+			} else
+			{
+				FPU registers[reg] = fp_pow10(src);
+			}
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x14:		/* FLOGN */
 			fpu_debug(("FLOGN %.04f\n",(double)src));
-			FPU registers[reg] = fp_log (src);
+			if (isinf(src) && !isneg(src))
+				make_inf(FPU registers[reg], false);
+			else
+				FPU registers[reg] = fp_log (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x15:		/* FLOG10 */
 			fpu_debug(("FLOG10 %.04f\n",(double)src));
-			FPU registers[reg] = fp_log10 (src);
+			if (isinf(src) && !isneg(src))
+				make_inf(FPU registers[reg], false);
+			else
+				FPU registers[reg] = fp_log10 (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x16:		/* FLOG2 */
 			fpu_debug(("FLOG2 %.04f\n",(double)src));
-			FPU registers[reg] = fp_log (src) / fp_log (2.0);
+			if (isinf(src) && !isneg(src))
+				make_inf(FPU registers[reg], false);
+			else
+				FPU registers[reg] = fp_log2 (src);
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x18:		/* FABS */
@@ -1943,12 +2084,21 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			break;
 		case 0x19:		/* FCOSH */
 			fpu_debug(("FCOSH %.04f\n",(double)src));
-			FPU registers[reg] = fp_cosh(src);
+			if (isinf(src))
+			{
+				make_inf(FPU registers[reg], false);
+			} else
+			{
+				FPU registers[reg] = fp_cosh(src);
+			}
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x1a:		/* FNEG */
 			fpu_debug(("FNEG %.04f\n",(double)src));
-			FPU registers[reg] = -src;
+			if (iszero(src))
+				make_zero(FPU registers[reg], !isneg(src));
+			else
+				FPU registers[reg] = -src;
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x1c:		/* FACOS */
@@ -1964,20 +2114,24 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 		case 0x1e:		/* FGETEXP */
 			fpu_debug(("FGETEXP %.04f\n",(double)src));
 			if( isinf(src) ) {
-				make_nan( FPU registers[reg] );
+				make_nan( FPU registers[reg], isneg(src) );
+			}
+			else if( iszero(src) ) {
+				make_zero(FPU registers[reg], isneg(src));
 			}
 			else {
+				/* FIXME: subnormals not supported */
 				FPU registers[reg] = fast_fgetexp( src );
 			}
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x1f:		/* FGETMAN */
 			fpu_debug(("FGETMAN %.04f\n",(double)src));
-			if( src == 0 ) {
-				FPU registers[reg] = 0;
+			if( iszero(src)) {
+				make_zero(FPU registers[reg], isneg(src));
 			}
-			else if( isinf(src) ) {
-				make_nan( FPU registers[reg] );
+			else if( isinf(src) || isnan(src) ) {
+				make_nan( FPU registers[reg], 0 );
 			}
 			else {
 				FPU registers[reg] = src;
@@ -1987,7 +2141,28 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			break;
 		case 0x20:		/* FDIV */
 			fpu_debug(("FDIV %.04f\n",(double)src));
-			FPU registers[reg] /= src;
+			if (isnan(src) || isnan(FPU registers[reg]))
+			{
+				make_nan(FPU registers[reg], false);
+			} else if (isinf(src))
+			{
+				if (isinf(FPU registers[reg]))
+					make_nan(FPU registers[reg], false);
+				else
+					make_zero(FPU registers[reg], isneg(src) != isneg(FPU registers[reg]));
+			} else if (isinf(FPU registers[reg]))
+			{
+				if (isinf(src))
+					make_nan(FPU registers[reg], false);
+				else
+					make_inf(FPU registers[reg], isneg(src) != isneg(FPU registers[reg]));
+			} else if (iszero(FPU registers[reg]) && !iszero(src))
+			{
+				make_zero(FPU registers[reg], isneg(FPU registers[reg]) != isneg(src));
+			} else
+			{
+				FPU registers[reg] /= src;
+			}
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x21:		/* FMOD */
@@ -2007,29 +2182,21 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			get_source_flags(src);
 			if (fl_dest.in_range && fl_source.in_range) {
 				FPU registers[reg] *= src;
+				if (unlikely(isinf(FPU registers[reg])))
+				{
+					make_inf(FPU registers[reg], isneg(FPU registers[reg]));
+				}
 			}
 			else if (fl_dest.nan || fl_source.nan || 
 					 (fl_dest.zero && fl_source.infinity) || 
 					 (fl_dest.infinity && fl_source.zero) ) {
-				make_nan( FPU registers[reg] );
+				make_nan( FPU registers[reg], fl_dest.negative );
 			}
 			else if (fl_dest.zero || fl_source.zero ) {
-				if ( (fl_dest.negative && !fl_source.negative) ||
-					(!fl_dest.negative && fl_source.negative) )  {
-					make_zero_negative(FPU registers[reg]);
-				}
-				else {
-					make_zero_positive(FPU registers[reg]);
-				}
+				make_zero(FPU registers[reg], fl_dest.negative != fl_source.negative);
 			}
 			else {
-				if ( (fl_dest.negative && !fl_source.negative) ||
-					(!fl_dest.negative && fl_source.negative) )  {
-					make_inf_negative(FPU registers[reg]);
-				}
-				else {
-					make_inf_positive(FPU registers[reg]);
-				}
+				make_inf(FPU registers[reg], fl_dest.negative != fl_source.negative);
 			}
 			make_fpsr(FPU registers[reg]);
 			break;
@@ -2063,8 +2230,26 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 #if defined(USE_LONG_DOUBLE) || defined(USE_QUAD_DOUBLE)
 				fp_declare_init_shape(sxp, extended);
 				sxp.value = FPU registers[reg];
-				sxp.ieee.exponent += scale_factor;
-				FPU registers[reg] = sxp.value;
+				int exp = sxp.ieee.exponent;
+				exp += scale_factor;
+				if (scale_factor >= FP_EXTENDED_EXP_MAX || exp >= FP_EXTENDED_EXP_MAX) /* overflow */
+				{
+					make_inf(FPU registers[reg], isneg(FPU registers[reg]));
+					FPU fpsr.exception_status |= FPSR_EXCEPTION_OVFL;
+				} else if (scale_factor < -FP_EXTENDED_EXP_MAX || exp <= -64) /* underflow */
+				{
+					make_zero(FPU registers[reg], isneg(FPU registers[reg]));
+					FPU fpsr.exception_status |= FPSR_EXCEPTION_UNFL;
+				} else if (exp >= 0) /* normal result */
+				{
+					sxp.ieee.exponent = exp;
+					FPU registers[reg] = sxp.value;
+				} else /* subnormal result */
+				{
+					exp += 64;
+					sxp.ieee.exponent = exp;
+					sxp.value = sxp.value * 5.421010862427522170037e-20L; /* 2^-64 */
+				}
 #else
 				fp_declare_init_shape(sxp, double);
 				sxp.value = FPU registers[reg];
@@ -2079,9 +2264,12 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 				FPU registers[reg] = sxp.value;
 #endif
 			}
-			else if (fl_source.infinity) {
+			else if (fl_source.infinity || fl_source.nan) {
 				// Returns NaN for any Infinity source
-				make_nan( FPU registers[reg] );
+				make_nan( FPU registers[reg], fl_source.negative );
+			} else {
+				// source was zero, or dest was inf or nan
+				// in either case, dest is unchanged
 			}
 			make_fpsr(FPU registers[reg]);
 			break;
@@ -2092,12 +2280,52 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			break;
 		case 0x28:		/* FSUB */
 			fpu_debug(("FSUB %.04f\n",(double)src));
-			FPU registers[reg] -= src;
+			if (isnan(src) || isnan(FPU registers[reg]))
+			{
+				make_nan(FPU registers[reg], false);
+			} else if (isinf(src))
+			{
+				if (isinf(FPU registers[reg]) && isneg(src) == isneg(FPU registers[reg]))
+					make_nan(FPU registers[reg], false);
+				else
+					make_inf(FPU registers[reg], isneg(src));
+			} else if (isinf(FPU registers[reg]))
+			{
+				if (isinf(src) && isneg(src) == isneg(FPU registers[reg]))
+					make_nan(FPU registers[reg], false);
+				else
+					make_inf(FPU registers[reg], isneg(FPU registers[reg]));
+			} else
+			{
+				FPU registers[reg] -= src;
+			}
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x22:		/* FADD */
 			fpu_debug(("FADD %.04f\n",(double)src));
-			FPU registers[reg] += src;
+			/*
+			 * WTF. inf + some value generates NaN on x87,
+			 * but we need inf in most cases
+			 */
+			if (isnan(src) || isnan(FPU registers[reg]))
+			{
+				make_nan(FPU registers[reg], false);
+			} else if (isinf(src))
+			{
+				if (isinf(FPU registers[reg]) && isneg(src) != isneg(FPU registers[reg]))
+					make_nan(FPU registers[reg], false);
+				else
+					make_inf(FPU registers[reg], isneg(src));
+			} else if (isinf(FPU registers[reg]))
+			{
+				if (isinf(src) && isneg(src) != isneg(FPU registers[reg]))
+					make_nan(FPU registers[reg], false);
+				else
+					make_inf(FPU registers[reg], isneg(FPU registers[reg]));
+			} else
+			{
+				FPU registers[reg] += src;
+			}
 			make_fpsr(FPU registers[reg]);
 			break;
 		case 0x30:		/* FSINCOS */
@@ -2110,6 +2338,7 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 		case 0x37:
 			fpu_debug(("FSINCOS %.04f\n",(double)src));
 			// Cosine must be calculated first if same register
+			// note: no need to use special sincos() function here; compiler will optimize that anyway
 			FPU registers[extra & 7] = fp_cos(src);
 			FPU registers[reg] = fp_sin (src);
 			// Set FPU fpsr according to the sine result
@@ -2127,7 +2356,7 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 			}
 			else if (isinf(src))
 				make_fpsr(-src);
-      else
+			else
 				make_fpsr(FPU registers[reg] - src);
 			break;
 		case 0x3a:		/* FTST */
@@ -2152,6 +2381,27 @@ void FFPU fpuop_arithmetic(uae_u32 opcode, uae_u32 extra)
 	dump_registers( "END  ");
 }
 
+
+void fpu_set_fpsr(uae_u32 new_fpsr)
+{
+	set_fpsr(new_fpsr);
+}
+
+uae_u32 fpu_get_fpsr(void)
+{
+	return get_fpsr();
+}
+
+void fpu_set_fpcr(uae_u32 new_fpcr)
+{
+	set_fpcr(new_fpcr);
+}
+
+uae_u32 fpu_get_fpcr(void)
+{
+	return get_fpcr();
+}
+
 /* -------------------------- Initialization -------------------------- */
 
 PRIVATE uae_u8 m_fpu_state_original[108]; // 90/94/108
@@ -2188,7 +2438,7 @@ PUBLIC void FFPU fpu_init (bool integral_68040)
 	FPU result = 1;
 	
 	for (int i = 0; i < 8; i++)
-		make_nan(FPU registers[i]);
+		make_nan(FPU registers[i], false);
 }
 
 PUBLIC void FFPU fpu_exit (void)
