@@ -42,6 +42,9 @@
 
 #ifndef SAHF_SETO_PROFITABLE
 
+/*
+ * Machine dependent structure for holding the 68k CCR flags
+ */
 /* PUSH/POP instructions are naturally 64-bit sized on x86-64, thus
    unsigned long hereunder is either 64-bit or 32-bit wide depending
    on the target.  */
@@ -55,55 +58,86 @@ struct flag_struct {
 #endif
 };
 
-#define FLAGVAL_Z	0x40
-#define FLAGVAL_N	0x80
+/*
+ * The bits in the cznv field in the above structure are assigned to
+ * allow the easy mirroring of the x86 rFLAGS register.
+ *
+ * The 68k CZNV flags are thus assigned in cznv as:
+ *
+ * 76543210  FEDCBA98 --------- ---------
+ * SZxxxxxC  xxxxVxxx xxxxxxxxx xxxxxxxxx
+ */
 
-#define SET_ZFLG(y)	(regflags.cznv = (((uae_u32)regflags.cznv) & ~0x40) | (((y) & 1) << 6))
-#define SET_CFLG(y)	(regflags.cznv = (((uae_u32)regflags.cznv) & ~1) | ((y) & 1))
-#define SET_VFLG(y)	(regflags.cznv = (((uae_u32)regflags.cznv) & ~0x800) | (((y) & 1) << 11))
-#define SET_NFLG(y)	(regflags.cznv = (((uae_u32)regflags.cznv) & ~0x80) | (((y) & 1) << 7))
-#define SET_XFLG(y)	(regflags.x = (y))
+#define FLAGBIT_N   7
+#define FLAGBIT_Z   6
+#define FLAGBIT_C   0
+#define FLAGBIT_V   11
+#define FLAGBIT_X   FLAGBIT_C /* should be in the same position as the x flag */
 
-#define GET_ZFLG	((regflags.cznv >> 6) & 1)
-#define GET_CFLG	(regflags.cznv & 1)
-#define GET_VFLG	((regflags.cznv >> 11) & 1)
-#define GET_NFLG	((regflags.cznv >> 7) & 1)
-#define GET_XFLG	(regflags.x & 1)
+#define FLAGVAL_N   (1 << FLAGBIT_N)
+#define FLAGVAL_Z   (1 << FLAGBIT_Z)
+#define FLAGVAL_C   (1 << FLAGBIT_C)
+#define FLAGVAL_V   (1 << FLAGBIT_V)
+#define FLAGVAL_X   (1 << FLAGBIT_X)
 
-#define CLEAR_CZNV	(regflags.cznv = 0)
-#define GET_CZNV	(regflags.cznv)
-#define IOR_CZNV(X)	(regflags.cznv |= (X))
-#define SET_CZNV(X)	(regflags.cznv = (X))
+#define SET_ZFLG(y) (regflags.cznv = (((uae_u32)regflags.cznv) & ~FLAGVAL_Z) | (((y) & 1) << FLAGBIT_Z))
+#define SET_CFLG(y) (regflags.cznv = (((uae_u32)regflags.cznv) & ~FLAGVAL_C) | (((y) & 1) << FLAGBIT_C))
+#define SET_VFLG(y) (regflags.cznv = (((uae_u32)regflags.cznv) & ~FLAGVAL_V) | (((y) & 1) << FLAGBIT_V))
+#define SET_NFLG(y) (regflags.cznv = (((uae_u32)regflags.cznv) & ~FLAGVAL_N) | (((y) & 1) << FLAGBIT_N))
+#define SET_XFLG(y) (regflags.x = ((y) & 1) << FLAGBIT_X)
 
-#define COPY_CARRY	(regflags.x = regflags.cznv)
+#define GET_ZFLG()	((regflags.cznv >> FLAGBIT_Z) & 1)
+#define GET_CFLG()	((regflags.cznv >> FLAGBIT_C) & 1)
+#define GET_VFLG()	((regflags.cznv >> FLAGBIT_V) & 1)
+#define GET_NFLG()	((regflags.cznv >> FLAGBIT_N) & 1)
+#define GET_XFLG()	((regflags.x    >> FLAGBIT_X) & 1)
+
+#define CLEAR_CZNV()	(regflags.cznv = 0)
+#define GET_CZNV()		(regflags.cznv)
+#define IOR_CZNV(X)		(regflags.cznv |= (X))
+#define SET_CZNV(X)		(regflags.cznv = (X))
+
+#define COPY_CARRY()	(regflags.x = regflags.cznv >> (FLAGBIT_C - FLAGBIT_X))
 
 extern struct flag_struct regflags __asm__ ("regflags");
 
+/*
+ * Test CCR condition
+ */
 static inline int cctrue(int cc)
 {
     uae_u32 cznv = regflags.cznv;
-    switch(cc){
-     case 0: return 1;                       /* T */
-     case 1: return 0;                       /* F */
-     case 2: return (cznv & 0x41) == 0; /* !GET_CFLG && !GET_ZFLG;  HI */
-     case 3: return (cznv & 0x41) != 0; /* GET_CFLG || GET_ZFLG;    LS */
-     case 4: return (cznv & 1) == 0;        /* !GET_CFLG;               CC */
-     case 5: return (cznv & 1) != 0;           /* GET_CFLG;                CS */
-     case 6: return (cznv & 0x40) == 0; /* !GET_ZFLG;               NE */
-     case 7: return (cznv & 0x40) != 0; /* GET_ZFLG;                EQ */
-     case 8: return (cznv & 0x800) == 0;/* !GET_VFLG;               VC */
-     case 9: return (cznv & 0x800) != 0;/* GET_VFLG;                VS */
-     case 10:return (cznv & 0x80) == 0; /* !GET_NFLG;               PL */
-     case 11:return (cznv & 0x80) != 0; /* GET_NFLG;                MI */
-     case 12:return (((cznv << 4) ^ cznv) & 0x800) == 0; /* GET_NFLG == GET_VFLG;             GE */
-     case 13:return (((cznv << 4) ^ cznv) & 0x800) != 0;/* GET_NFLG != GET_VFLG;             LT */
-     case 14:
-	cznv &= 0x8c0;
-	return (((cznv << 4) ^ cznv) & 0x840) == 0; /* !GET_ZFLG && (GET_NFLG == GET_VFLG);  GT */
-     case 15:
-	cznv &= 0x8c0;
-	return (((cznv << 4) ^ cznv) & 0x840) != 0; /* GET_ZFLG || (GET_NFLG != GET_VFLG);   LE */
+
+    switch (cc) {
+    case 0:  return 1;                              /*              T  */
+    case 1:  return 0;                              /*              F  */
+    case 2:  return (cznv & (FLAGVAL_C | FLAGVAL_Z)) == 0;              /* !CFLG && !ZFLG       HI */
+    case 3:  return (cznv & (FLAGVAL_C | FLAGVAL_Z)) != 0;              /*  CFLG || ZFLG        LS */
+    case 4:  return (cznv & FLAGVAL_C) == 0;                    /* !CFLG            CC */
+    case 5:  return (cznv & FLAGVAL_C) != 0;                    /*  CFLG            CS */
+    case 6:  return (cznv & FLAGVAL_Z) == 0;                    /* !ZFLG            NE */
+    case 7:  return (cznv & FLAGVAL_Z) != 0;                    /*  ZFLG            EQ */
+    case 8:  return (cznv & FLAGVAL_V) == 0;                    /* !VFLG            VC */
+    case 9:  return (cznv & FLAGVAL_V) != 0;                    /*  VFLG            VS */
+    case 10: return (cznv & FLAGVAL_N) == 0;                    /* !NFLG            PL */
+    case 11: return (cznv & FLAGVAL_N) != 0;                    /*  NFLG            MI */
+#if FLAGBIT_N > FLAGBIT_V
+    case 12: return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & FLAGVAL_N) == 0;  /*  NFLG == VFLG        GE */
+    case 13: return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & FLAGVAL_N) != 0;  /*  NFLG != VFLG        LT */
+    case 14: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* !ZFLG && (NFLG == VFLG)   GT */
+         return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & (FLAGVAL_N | FLAGVAL_Z)) == 0;
+    case 15: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* ZFLG || (NFLG != VFLG)   LE */
+         return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & (FLAGVAL_N | FLAGVAL_Z)) != 0;
+#else
+    case 12: return (((cznv << (FLAGBIT_V - FLAGBIT_N)) ^ cznv) & FLAGVAL_V) == 0;  /*  NFLG == VFLG        GE */
+    case 13: return (((cznv << (FLAGBIT_V - FLAGBIT_N)) ^ cznv) & FLAGVAL_V) != 0;  /*  NFLG != VFLG        LT */
+    case 14: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* !ZFLG && (NFLG == VFLG)   GT */
+         return (((cznv << (FLAGBIT_V - FLAGBIT_N)) ^ cznv) & (FLAGVAL_V | FLAGVAL_Z)) == 0;
+    case 15: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* ZFLG || (NFLG != VFLG)   LE */
+         return (((cznv << (FLAGBIT_V - FLAGBIT_N)) ^ cznv) & (FLAGVAL_V | FLAGVAL_Z)) != 0;
+#endif
     }
+    abort ();
     return 0;
 }
 
@@ -130,7 +164,7 @@ static inline int cctrue(int cc)
 			"pushf\n\t" \
 			"pop %0\n\t" \
 			: "=rm" (regflags.cznv), "=r" (v) : "rmi" (s), "1" (d) : "memory", "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while (0)
 
 #define optflag_addw(v, s, d) do { \
@@ -138,7 +172,7 @@ static inline int cctrue(int cc)
 			"pushf\n\t" \
 			"pop %0\n\t" \
 			: "=rm" (regflags.cznv), "=r" (v) : "rmi" (s), "1" (d) : "memory", "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while (0)
 
 #define optflag_addb(v, s, d) do { \
@@ -146,7 +180,7 @@ static inline int cctrue(int cc)
 			"pushf\n\t" \
 			"pop %0\n\t" \
 			: "=r" (regflags.cznv), "=q" (v) : "qmi" (s), "1" (d) : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while (0)
 
 #define optflag_subl(v, s, d) do { \
@@ -154,7 +188,7 @@ static inline int cctrue(int cc)
 			"pushf\n\t" \
 			"pop %0\n\t" \
 			: "=rm" (regflags.cznv), "=r" (v) : "rmi" (s), "1" (d) : "memory", "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while (0)
 
 #define optflag_subw(v, s, d) do { \
@@ -162,7 +196,7 @@ static inline int cctrue(int cc)
 			"pushf\n\t" \
 			"pop %0\n\t" \
 			: "=rm" (regflags.cznv), "=r" (v) : "rmi" (s), "1" (d) : "memory", "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while (0)
 
 #define optflag_subb(v, s, d) do { \
@@ -170,7 +204,7 @@ static inline int cctrue(int cc)
 			"pushf\n\t" \
 			"pop %0\n\t" \
 			: "=rm" (regflags.cznv), "=q" (v) : "qmi" (s), "1" (d) : "memory", "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while (0)
 
 #define optflag_cmpl(s, d) \
@@ -191,63 +225,101 @@ static inline int cctrue(int cc)
 			"pop %0\n\t" \
 			: "=rm" (regflags.cznv) : "qmi" (s), "q" (d) : "memory", "cc")
 
-#else
+#else /* !SAHF_SETO_PROFITABLE */
 
+/*
+ * Machine dependent structure for holding the 68k CCR flags
+ */
 struct flag_struct {
-    uae_u32 cznv;
-    uae_u32 x;
+	uae_u32 cznv;
+	uae_u32 x;
 };
 
-#define FLAGVAL_Z	0x4000
-#define FLAGVAL_N	0x8000
-
-#define SET_ZFLG(y)	(regflags.cznv = (regflags.cznv & ~0x4000) | (((y) & 1) << 14))
-#define SET_CFLG(y)	(regflags.cznv = (regflags.cznv & ~0x100) | (((y) & 1) << 8))
-#define SET_VFLG(y)	(regflags.cznv = (regflags.cznv & ~0x1) | (((y) & 1)))
-#define SET_NFLG(y)	(regflags.cznv = (regflags.cznv & ~0x8000) | (((y) & 1) << 15))
-#define SET_XFLG(y)	(regflags.x = (y))
-
-#define GET_ZFLG	((regflags.cznv >> 14) & 1)
-#define GET_CFLG	((regflags.cznv >> 8) & 1)
-#define GET_VFLG	((regflags.cznv >> 0) & 1)
-#define GET_NFLG	((regflags.cznv >> 15) & 1)
-#define GET_XFLG	(regflags.x & 1)
-
-#define CLEAR_CZNV 	(regflags.cznv = 0)
-#define GET_CZNV	(regflags.cznv)
-#define IOR_CZNV(X)	(regflags.cznv |= (X))
-#define SET_CZNV(X)	(regflags.cznv = (X))
-
-#define COPY_CARRY	(regflags.x = (regflags.cznv)>>8)
-
 extern struct flag_struct regflags __asm__ ("regflags");
 
+/*
+ * The bits in the cznv field in the above structure are assigned to
+ * allow the easy mirroring of the x86 condition flags. (For example,
+ * from the AX register - the x86 overflow flag can be copied to AL
+ * with a setto %AL instr and the other flags copied to AH with an
+ * lahf instr).
+ *
+ * The 68k CZNV flags are thus assigned in cznv as:
+ *
+ * <--AL-->  <--AH-->
+ * 76543210  FEDCBA98 --------- ---------
+ * xxxxxxxV  NZxxxxxC xxxxxxxxx xxxxxxxxx
+ */
+
+#define FLAGBIT_N   15
+#define FLAGBIT_Z   14
+#define FLAGBIT_C   8
+#define FLAGBIT_V   0
+#define FLAGBIT_X   FLAGBIT_C /* should be in the same position as the x flag */
+
+#define FLAGVAL_N   (1 << FLAGBIT_N)
+#define FLAGVAL_Z   (1 << FLAGBIT_Z)
+#define FLAGVAL_C   (1 << FLAGBIT_C)
+#define FLAGVAL_V   (1 << FLAGBIT_V)
+#define FLAGVAL_X   (1 << FLAGBIT_X)
+
+#define SET_ZFLG(y) (regflags.cznv = (((uae_u32)regflags.cznv) & ~FLAGVAL_Z) | (((y) & 1) << FLAGBIT_Z))
+#define SET_CFLG(y) (regflags.cznv = (((uae_u32)regflags.cznv) & ~FLAGVAL_C) | (((y) & 1) << FLAGBIT_C))
+#define SET_VFLG(y) (regflags.cznv = (((uae_u32)regflags.cznv) & ~FLAGVAL_V) | (((y) & 1) << FLAGBIT_V))
+#define SET_NFLG(y) (regflags.cznv = (((uae_u32)regflags.cznv) & ~FLAGVAL_N) | (((y) & 1) << FLAGBIT_N))
+#define SET_XFLG(y) (regflags.x    = ((y) & 1) << FLAGBIT_X)
+
+#define GET_ZFLG()	((regflags.cznv >> FLAGBIT_Z) & 1)
+#define GET_CFLG()	((regflags.cznv >> FLAGBIT_C) & 1)
+#define GET_VFLG()	((regflags.cznv >> FLAGBIT_V) & 1)
+#define GET_NFLG()	((regflags.cznv >> FLAGBIT_N) & 1)
+#define GET_XFLG()	((regflags.x    >> FLAGBIT_X) & 1)
+
+#define CLEAR_CZNV()	(regflags.cznv = 0)
+#define GET_CZNV()		(regflags.cznv)
+#define IOR_CZNV(X)		(regflags.cznv |= (X))
+#define SET_CZNV(X)		(regflags.cznv = (X))
+
+#define COPY_CARRY()	(regflags.x = regflags.cznv >> (FLAGBIT_C - FLAGBIT_X))
+
+
+/*
+ * Test CCR condition
+ */
 static inline int cctrue(int cc)
 {
     uae_u32 cznv = regflags.cznv;
-    switch(cc){
-     case 0: return 1;                       /* T */
-     case 1: return 0;                       /* F */
-     case 2: return (cznv & 0x4100) == 0; /* !GET_CFLG && !GET_ZFLG;  HI */
-     case 3: return (cznv & 0x4100) != 0; /* GET_CFLG || GET_ZFLG;    LS */
-     case 4: return (cznv & 0x100) == 0;  /* !GET_CFLG;               CC */
-     case 5: return (cznv & 0x100) != 0;  /* GET_CFLG;                CS */
-     case 6: return (cznv & 0x4000) == 0; /* !GET_ZFLG;               NE */
-     case 7: return (cznv & 0x4000) != 0; /* GET_ZFLG;                EQ */
-     case 8: return (cznv & 0x01) == 0;   /* !GET_VFLG;               VC */
-     case 9: return (cznv & 0x01) != 0;   /* GET_VFLG;                VS */
-     case 10:return (cznv & 0x8000) == 0; /* !GET_NFLG;               PL */
-     case 11:return (cznv & 0x8000) != 0; /* GET_NFLG;                MI */
-     case 12:return (((cznv << 15) ^ cznv) & 0x8000) == 0; /* GET_NFLG == GET_VFLG;             GE */
-     case 13:return (((cznv << 15) ^ cznv) & 0x8000) != 0;/* GET_NFLG != GET_VFLG;             LT */
-     case 14:
-	cznv &= 0xc001;
-	return (((cznv << 15) ^ cznv) & 0xc000) == 0; /* !GET_ZFLG && (GET_NFLG == GET_VFLG);  GT */
-     case 15:
-	cznv &= 0xc001;
-	return (((cznv << 15) ^ cznv) & 0xc000) != 0; /* GET_ZFLG || (GET_NFLG != GET_VFLG);   LE */
+
+    switch (cc) {
+    case 0:  return 1;                              /*              T  */
+    case 1:  return 0;                              /*              F  */
+    case 2:  return (cznv & (FLAGVAL_C | FLAGVAL_Z)) == 0;              /* !CFLG && !ZFLG       HI */
+    case 3:  return (cznv & (FLAGVAL_C | FLAGVAL_Z)) != 0;              /*  CFLG || ZFLG        LS */
+    case 4:  return (cznv & FLAGVAL_C) == 0;                    /* !CFLG            CC */
+    case 5:  return (cznv & FLAGVAL_C) != 0;                    /*  CFLG            CS */
+    case 6:  return (cznv & FLAGVAL_Z) == 0;                    /* !ZFLG            NE */
+    case 7:  return (cznv & FLAGVAL_Z) != 0;                    /*  ZFLG            EQ */
+    case 8:  return (cznv & FLAGVAL_V) == 0;                    /* !VFLG            VC */
+    case 9:  return (cznv & FLAGVAL_V) != 0;                    /*  VFLG            VS */
+    case 10: return (cznv & FLAGVAL_N) == 0;                    /* !NFLG            PL */
+    case 11: return (cznv & FLAGVAL_N) != 0;                    /*  NFLG            MI */
+#if FLAGBIT_N > FLAGBIT_V
+    case 12: return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & FLAGVAL_N) == 0;  /*  NFLG == VFLG        GE */
+    case 13: return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & FLAGVAL_N) != 0;  /*  NFLG != VFLG        LT */
+    case 14: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* !ZFLG && (NFLG == VFLG)   GT */
+         return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & (FLAGVAL_N | FLAGVAL_Z)) == 0;
+    case 15: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* ZFLG || (NFLG != VFLG)   LE */
+         return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & (FLAGVAL_N | FLAGVAL_Z)) != 0;
+#else
+    case 12: return (((cznv << (FLAGBIT_V - FLAGBIT_N)) ^ cznv) & FLAGVAL_V) == 0;  /*  NFLG == VFLG        GE */
+    case 13: return (((cznv << (FLAGBIT_V - FLAGBIT_N)) ^ cznv) & FLAGVAL_V) != 0;  /*  NFLG != VFLG        LT */
+    case 14: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* !ZFLG && (NFLG == VFLG)   GT */
+         return (((cznv << (FLAGBIT_V - FLAGBIT_N)) ^ cznv) & (FLAGVAL_V | FLAGVAL_Z)) == 0;
+    case 15: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* ZFLG || (NFLG != VFLG)   LE */
+         return (((cznv << (FLAGBIT_V - FLAGBIT_N)) ^ cznv) & (FLAGVAL_V | FLAGVAL_Z)) != 0;
+#endif
     }
-    abort();
+    abort ();
     return 0;
 }
 
@@ -291,7 +363,7 @@ static inline int cctrue(int cc)
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
 			: "=r" (v) : "rmi" (s), "0" (d) : "%eax","cc","memory"); \
-			COPY_CARRY; \
+			COPY_CARRY(); \
 	} while (0)
 
 #define optflag_addw(v, s, d) do { \
@@ -301,7 +373,7 @@ static inline int cctrue(int cc)
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
 			: "=r" (v) : "rmi" (s), "0" (d) : "%eax","cc","memory"); \
-			COPY_CARRY; \
+			COPY_CARRY(); \
     } while (0)
 
 #define optflag_addb(v, s, d) do { \
@@ -311,7 +383,7 @@ static inline int cctrue(int cc)
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
 			: "=q" (v) : "qmi" (s), "0" (d) : "%eax","cc","memory"); \
-			COPY_CARRY; \
+			COPY_CARRY(); \
     } while (0)
 
 #define optflag_subl(v, s, d) do { \
@@ -321,7 +393,7 @@ static inline int cctrue(int cc)
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
 			: "=r" (v) : "rmi" (s), "0" (d) : "%eax","cc","memory"); \
-			COPY_CARRY; \
+			COPY_CARRY(); \
     } while (0)
 
 #define optflag_subw(v, s, d) do { \
@@ -331,7 +403,7 @@ static inline int cctrue(int cc)
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
 			: "=r" (v) : "rmi" (s), "0" (d) : "%eax","cc","memory"); \
-			COPY_CARRY; \
+			COPY_CARRY(); \
     } while (0)
 
 #define optflag_subb(v, s, d) do { \
@@ -341,7 +413,7 @@ static inline int cctrue(int cc)
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
 			: "=q" (v) : "qmi" (s), "0" (d) : "%eax","cc","memory"); \
-			COPY_CARRY; \
+			COPY_CARRY(); \
     } while (0)
 
 #define optflag_cmpl(s, d) \
@@ -368,66 +440,76 @@ static inline int cctrue(int cc)
 			"movb %%ah,regflags+1\n\t" \
 			: : "qmi" (s), "q" (d) : "%eax","cc","memory")
 
-#endif
+#endif /* SAHF_SETO_PROFITABLE */
 
 #elif defined(CPU_arm) && defined(ARM_ASSEMBLY)
 
+/*
+ * Machine dependent structure for holding the 68k CCR flags
+ */
 struct flag_struct {
 	uae_u32 nzcv;
 	uae_u32 x;
 };
 
-#define FLAGVAL_Q       0x08000000
-#define FLAGVAL_V       0x10000000
-#define FLAGVAL_C       0x20000000
-#define FLAGVAL_Z       0x40000000
-#define FLAGVAL_N       0x80000000
+#define FLAGBIT_N   31
+#define FLAGBIT_Z   30
+#define FLAGBIT_C   29
+#define FLAGBIT_V   28
+#define FLAGBIT_X   FLAGBIT_C /* must be in the same position in as x flag */
 
-#define SET_NFLG(y)     (regflags.nzcv = (regflags.nzcv & ~0x80000000) | (((y) & 1) << 31))
-#define SET_ZFLG(y)     (regflags.nzcv = (regflags.nzcv & ~0x40000000) | (((y) & 1) << 30))
-#define SET_CFLG(y)     (regflags.nzcv = (regflags.nzcv & ~0x20000000) | (((y) & 1) << 29))
-#define SET_VFLG(y)     (regflags.nzcv = (regflags.nzcv & ~0x10000000) | (((y) & 1) << 28))
-#define SET_XFLG(y)     (regflags.x = (y))
+#define FLAGVAL_N   (1 << FLAGBIT_N)
+#define FLAGVAL_Z   (1 << FLAGBIT_Z)
+#define FLAGVAL_C   (1 << FLAGBIT_C)
+#define FLAGVAL_V   (1 << FLAGBIT_V)
+#define FLAGVAL_X   (1 << FLAGBIT_X)
 
-#define GET_NFLG        ((regflags.nzcv >> 31) & 1)
-#define GET_ZFLG        ((regflags.nzcv >> 30) & 1)
-#define GET_CFLG        ((regflags.nzcv >> 29) & 1)
-#define GET_VFLG        ((regflags.nzcv >> 28) & 1)
-#define GET_XFLG        (regflags.x & 1)
+#define SET_NFLG(y)		(regflags.nzcv = (regflags.nzcv & ~FLAGVAL_N) | (((y) & 1) << FLAGBIT_N))
+#define SET_ZFLG(y)		(regflags.nzcv = (regflags.nzcv & ~FLAGVAL_Z) | (((y) & 1) << FLAGBIT_Z))
+#define SET_CFLG(y)		(regflags.nzcv = (regflags.nzcv & ~FLAGVAL_C) | (((y) & 1) << FLAGBIT_C))
+#define SET_VFLG(y)		(regflags.nzcv = (regflags.nzcv & ~FLAGVAL_V) | (((y) & 1) << FLAGBIT_V))
+#define SET_XFLG(y)		(regflags.x = ((y) & 1) << FLAGBIT_X)
 
-#define CLEAR_CZNV      (regflags.nzcv = 0)
-#define GET_CZNV        (regflags.nzcv)
-#define IOR_CZNV(X)     (regflags.nzcv |= (X))
-#define SET_CZNV(X)     (regflags.nzcv = (X))
+#define GET_NFLG()		((regflags.nzcv >> FLAGBIT_N) & 1)
+#define GET_ZFLG()		((regflags.nzcv >> FLAGBIT_Z) & 1)
+#define GET_CFLG()		((regflags.nzcv >> FLAGBIT_C) & 1)
+#define GET_VFLG()		((regflags.nzcv >> FLAGBIT_V) & 1)
+#define GET_XFLG()		((regflags.x    >> FLAGBIT_X) & 1)
 
-#define COPY_CARRY      (regflags.x = (regflags.nzcv)>>29)
+#define CLEAR_CZNV()	(regflags.nzcv = 0)
+#define GET_CZNV()		(regflags.nzcv)
+#define IOR_CZNV(X)		(regflags.nzcv |= (X))
+#define SET_CZNV(X)		(regflags.nzcv = (X))
+
+#define COPY_CARRY()	(regflags.x = regflags.nzcv >> (FLAGBIT_C - FLAGBIT_X))
 
 extern struct flag_struct regflags __asm__ ("regflags");
 
+/*
+ * Test CCR condition
+ */
 static inline int cctrue(int cc)
 {
     unsigned int nzcv = regflags.nzcv;
     switch(cc){
      case 0: return 1;                       /* T */
      case 1: return 0;                       /* F */
-     case 2: return (nzcv & 0x60000000) == 0; /* !GET_CFLG && !GET_ZFLG;  HI */
-     case 3: return (nzcv & 0x60000000) != 0; /* GET_CFLG || GET_ZFLG;    LS */
-     case 4: return (nzcv & 0x20000000) == 0; /* !GET_CFLG;               CC */
-     case 5: return (nzcv & 0x20000000) != 0; /* GET_CFLG;                CS */
-     case 6: return (nzcv & 0x40000000) == 0; /* !GET_ZFLG;               NE */
-     case 7: return (nzcv & 0x40000000) != 0; /* GET_ZFLG;                EQ */
-     case 8: return (nzcv & 0x10000000) == 0; /* !GET_VFLG;               VC */
-     case 9: return (nzcv & 0x10000000) != 0; /* GET_VFLG;                VS */
-     case 10:return (nzcv & 0x80000000) == 0; /* !GET_NFLG;               PL */
-     case 11:return (nzcv & 0x80000000) != 0; /* GET_NFLG;                MI */
-     case 12:return (((nzcv << 3) ^ nzcv) & 0x80000000) == 0; /* GET_NFLG == GET_VFLG;             GE */
-     case 13:return (((nzcv << 3) ^ nzcv) & 0x80000000) != 0; /* GET_NFLG != GET_VFLG;             LT */
-     case 14:
-        nzcv &= 0xd0000000;
-        return (((nzcv << 3) ^ nzcv) & 0xc0000000) == 0; /* !GET_ZFLG && (GET_NFLG == GET_VFLG);  GT */
-     case 15:
-        nzcv &= 0xd0000000;
-        return (((nzcv << 3) ^ nzcv) & 0xc0000000) != 0; /* GET_ZFLG || (GET_NFLG != GET_VFLG);   LE */
+     case 2: return (nzcv & (FLAGVAL_C | FLAGVAL_Z)) == 0; /* !GET_CFLG && !GET_ZFLG;  HI */
+     case 3: return (nzcv & (FLAGVAL_C | FLAGVAL_Z)) != 0; /* GET_CFLG || GET_ZFLG;    LS */
+     case 4: return (nzcv & FLAGVAL_C) == 0; /* !GET_CFLG;               CC */
+     case 5: return (nzcv & FLAGVAL_C) != 0; /* GET_CFLG;                CS */
+     case 6: return (nzcv & FLAGVAL_Z) == 0; /* !GET_ZFLG;               NE */
+     case 7: return (nzcv & FLAGVAL_Z) != 0; /* GET_ZFLG;                EQ */
+     case 8: return (nzcv & FLAGVAL_V) == 0; /* !GET_VFLG;               VC */
+     case 9: return (nzcv & FLAGVAL_V) != 0; /* GET_VFLG;                VS */
+     case 10:return (nzcv & FLAGVAL_N) == 0; /* !GET_NFLG;               PL */
+     case 11:return (nzcv & FLAGVAL_N) != 0; /* GET_NFLG;                MI */
+     case 12:return (((nzcv << (FLAGBIT_N - FLAGBIT_V)) ^ nzcv) & FLAGVAL_N) == 0; /* GET_NFLG == GET_VFLG;             GE */
+     case 13:return (((nzcv << (FLAGBIT_N - FLAGBIT_V)) ^ nzcv) & FLAGVAL_N) != 0; /* GET_NFLG != GET_VFLG;             LT */
+     case 14: nzcv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);
+        return (((nzcv << (FLAGBIT_N - FLAGBIT_V)) ^ nzcv) & (FLAGVAL_N | FLAGVAL_Z)) == 0; /* !GET_ZFLG && (GET_NFLG == GET_VFLG);  GT */
+     case 15: nzcv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);
+        return (((nzcv << (FLAGBIT_N - FLAGBIT_V)) ^ nzcv) & (FLAGVAL_N | FLAGVAL_Z)) != 0; /* GET_ZFLG || (GET_NFLG != GET_VFLG);   LE */
     }
     return 0;
 }
@@ -447,7 +529,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_subl(v, s, d) do { \
@@ -457,7 +539,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_cmpl(s, d) do { \
@@ -501,7 +583,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_addb(v, s, d) do { \
@@ -512,7 +594,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_subw(v, s, d) do { \
@@ -524,7 +606,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_subb(v, s, d) do { \
@@ -536,7 +618,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_cmpw(s, d) do { \
@@ -593,7 +675,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_addb(v, s, d) do { \
@@ -604,7 +686,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_subw(v, s, d) do { \
@@ -616,7 +698,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_subb(v, s, d) do { \
@@ -628,7 +710,7 @@ static inline int cctrue(int cc)
                         : [nzcv] "=r" (regflags.nzcv), [rv] "=r" (v) \
                         : [rs] "ri" (s), [rd] "1" (d) \
                         : "cc"); \
-    COPY_CARRY; \
+    COPY_CARRY(); \
     } while(0)
 
 #define optflag_cmpw(s, d) do { \
@@ -655,6 +737,9 @@ static inline int cctrue(int cc)
 
 #elif defined(CPU_sparc) && (defined(SPARC_V8_ASSEMBLY) || defined(SPARC_V9_ASSEMBLY))
 
+/*
+ * Machine dependent structure for holding the 68k CCR flags
+ */
 struct flag_struct {
     unsigned char nzvc;
     unsigned char x;
@@ -662,52 +747,62 @@ struct flag_struct {
 
 extern struct flag_struct regflags;
 
-#define FLAGVAL_Z	0x04
-#define FLAGVAL_N	0x08
+#define FLAGBIT_N   3
+#define FLAGBIT_Z   2
+#define FLAGBIT_V   1
+#define FLAGBIT_C   0
+#define FLAGBIT_X   FLAGBIT_C /* should be in the same position as the x flag */
 
-#define SET_ZFLG(y)	(regflags.nzvc = (regflags.nzvc & ~0x04) | (((y) & 1) << 2))
-#define SET_CFLG(y)	(regflags.nzvc = (regflags.nzvc & ~1) | ((y) & 1))
-#define SET_VFLG(y)	(regflags.nzvc = (regflags.nzvc & ~0x02) | (((y) & 1) << 1))
-#define SET_NFLG(y)	(regflags.nzvc = (regflags.nzvc & ~0x08) | (((y) & 1) << 3))
-#define SET_XFLG(y)	(regflags.x = (y))
+#define FLAGVAL_N   (1 << FLAGBIT_N)
+#define FLAGVAL_Z   (1 << FLAGBIT_Z)
+#define FLAGVAL_C   (1 << FLAGBIT_C)
+#define FLAGVAL_V   (1 << FLAGBIT_V)
+#define FLAGVAL_X   (1 << FLAGBIT_X)
 
-#define GET_ZFLG	((regflags.nzvc >> 2) & 1)
-#define GET_CFLG	(regflags.nzvc & 1)
-#define GET_VFLG	((regflags.nzvc >> 1) & 1)
-#define GET_NFLG	((regflags.nzvc >> 3) & 1)
-#define GET_XFLG	(regflags.x & 1)
+#define SET_ZFLG(y) (regflags.nzvc = (regflags.nzvc & ~FLAGVAL_Z) | (((y) & 1) << FLAGBIT_Z))
+#define SET_CFLG(y) (regflags.nzvc = (regflags.nzvc & ~FLAGVAL_C) | (((y) & 1) << FLAGBIT_C))
+#define SET_VFLG(y) (regflags.nzvc = (regflags.nzvc & ~FLAGVAL_V) | (((y) & 1) << FLAGBIT_V))
+#define SET_NFLG(y) (regflags.nzvc = (regflags.nzvc & ~FLAGVAL_V) | (((y) & 1) << FLAGBIT_N))
+#define SET_XFLG(y) (regflags.x = ((y) & 1) << FLAGBIT_X)
 
-#define CLEAR_CZNV 	(regflags.nzvc = 0)
-#define GET_CZNV	(reflags.nzvc)
-#define IOR_CZNV(X)	(refglags.nzvc |= (X))
-#define SET_CZNV(X)	(regflags.nzvc = (X))
+#define GET_ZFLG()	((regflags.nzvc >> FLAGBIT_Z) & 1)
+#define GET_CFLG()	((regflags.nzvc >> FLAGBIT_C) & 1)
+#define GET_VFLG()	((regflags.nzvc >> FLAGBIT_V) & 1)
+#define GET_NFLG()	((regflags.nzvc >> FLAGBIT_N) & 1)
+#define GET_XFLG()	((regflags.x    >> FLAGBIT_X) & 1)
 
-#define COPY_CARRY (regflags.x = regflags.nzvc)
+#define CLEAR_CZNV() 	(regflags.nzvc = 0)
+#define GET_CZNV()		(regflags.nzvc)
+#define IOR_CZNV(X)		(regflags.nzvc |= (X))
+#define SET_CZNV(X)		(regflags.nzvc = (X))
 
+#define COPY_CARRY()	(regflags.x = regflags.nzvc >> (FLAGBIT_C - FLAGBIT_X))
+
+/*
+ * Test CCR condition
+ */
 static inline int cctrue(int cc)
 {
     uae_u32 nzvc = regflags.nzvc;
-    switch(cc){
-     case 0: return 1;                       /* T */
-     case 1: return 0;                       /* F */
-     case 2: return (nzvc & 0x05) == 0; /* !GET_CFLG && !GET_ZFLG;  HI */
-     case 3: return (nzvc & 0x05) != 0; /* GET_CFLG || GET_ZFLG;    LS */
-     case 4: return (nzvc & 1) == 0;        /* !GET_CFLG;               CC */
-     case 5: return (nzvc & 1) != 0;           /* GET_CFLG;                CS */
-     case 6: return (nzvc & 0x04) == 0; /* !GET_ZFLG;               NE */
-     case 7: return (nzvc & 0x04) != 0; /* GET_ZFLG;                EQ */
-     case 8: return (nzvc & 0x02) == 0;/* !GET_VFLG;               VC */
-     case 9: return (nzvc & 0x02) != 0;/* GET_VFLG;                VS */
-     case 10:return (nzvc & 0x08) == 0; /* !GET_NFLG;               PL */
-     case 11:return (nzvc & 0x08) != 0; /* GET_NFLG;                MI */
-     case 12:return (((nzvc << 2) ^ nzvc) & 0x08) == 0; /* GET_NFLG == GET_VFLG;             GE */
-     case 13:return (((nzvc << 2) ^ nzvc) & 0x08) != 0;/* GET_NFLG != GET_VFLG;             LT */
-     case 14:
-	nzvc &= 0x0e;
-	return (((nzvc << 2) ^ nzvc) & 0x0c) == 0; /* !GET_ZFLG && (GET_NFLG == GET_VFLG);  GT */
-     case 15:
-	nzvc &= 0x0e;
-	return (((nzvc << 2) ^ nzvc) & 0x0c) != 0; /* GET_ZFLG || (GET_NFLG != GET_VFLG);   LE */
+    switch (cc) {
+    case 0:  return 1;                              /*              T  */
+    case 1:  return 0;                              /*              F  */
+    case 2:  return (cznv & (FLAGVAL_C | FLAGVAL_Z)) == 0;              /* !CFLG && !ZFLG       HI */
+    case 3:  return (cznv & (FLAGVAL_C | FLAGVAL_Z)) != 0;              /*  CFLG || ZFLG        LS */
+    case 4:  return (cznv & FLAGVAL_C) == 0;                    /* !CFLG            CC */
+    case 5:  return (cznv & FLAGVAL_C) != 0;                    /*  CFLG            CS */
+    case 6:  return (cznv & FLAGVAL_Z) == 0;                    /* !ZFLG            NE */
+    case 7:  return (cznv & FLAGVAL_Z) != 0;                    /*  ZFLG            EQ */
+    case 8:  return (cznv & FLAGVAL_V) == 0;                    /* !VFLG            VC */
+    case 9:  return (cznv & FLAGVAL_V) != 0;                    /*  VFLG            VS */
+    case 10: return (cznv & FLAGVAL_N) == 0;                    /* !NFLG            PL */
+    case 11: return (cznv & FLAGVAL_N) != 0;                    /*  NFLG            MI */
+    case 12: return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & FLAGVAL_N) == 0;  /*  NFLG == VFLG        GE */
+    case 13: return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & FLAGVAL_N) != 0;  /*  NFLG != VFLG        LT */
+    case 14: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* ZFLG && (NFLG == VFLG)   GT */
+         return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & (FLAGVAL_N | FLAGVAL_Z)) == 0;
+    case 15: cznv &= (FLAGVAL_N | FLAGVAL_Z | FLAGVAL_V);               /* ZFLG && (NFLG != VFLG)   LE */
+         return (((cznv << (FLAGBIT_N - FLAGBIT_V)) ^ cznv) & (FLAGVAL_N | FLAGVAL_Z)) != 0;
     }
     return 0;
 }
@@ -1309,6 +1404,9 @@ static inline uae_u32 sparc_v9_flag_addx_32(flag_struct *flags, uae_u32 src, uae
 
 #else
 
+/*
+ * Machine independent structure for holding the 68k CCR flags
+ */
 struct flag_struct {
     unsigned int c;
     unsigned int z;
@@ -1331,21 +1429,24 @@ extern struct flag_struct regflags;
 #define SET_ZFLG(x) (ZFLG = (x))
 #define SET_XFLG(x) (XFLG = (x))
 
-#define GET_CFLG CFLG
-#define GET_NFLG NFLG
-#define GET_VFLG VFLG
-#define GET_ZFLG ZFLG
-#define GET_XFLG XFLG
+#define GET_CFLG() CFLG
+#define GET_NFLG() NFLG
+#define GET_VFLG() VFLG
+#define GET_ZFLG() ZFLG
+#define GET_XFLG() XFLG
 
-#define CLEAR_CZNV do { \
+#define CLEAR_CZNV() do { \
  SET_CFLG (0); \
  SET_ZFLG (0); \
  SET_NFLG (0); \
  SET_VFLG (0); \
 } while (0)
 
-#define COPY_CARRY (SET_XFLG (GET_CFLG))
+#define COPY_CARRY() (SET_XFLG (GET_CFLG ()))
 
+/*
+ * Test CCR condition
+ */
 static inline int cctrue(const int cc)
 {
     switch(cc){
